@@ -47,7 +47,8 @@ const int    *EffectiveDisabledPages=    NULL,
              *EffectiveDisabledControls= NULL;
 BOOL          OpenCurSelection=          TRUE;
 
-#ifdef HAVE_OPENSSL
+/* On Windows we are supposed to have schannel, or we can have openssl */
+#if defined(_WIN32) || defined(HAVE_OPENSSL) 
 # define  SSL_DISABLED 0
 #else
 # define  SSL_DISABLED 1
@@ -111,6 +112,7 @@ MADB_OptionsMap OptionsMap[]= {
   {LASTPAGE, ckNullDate,               MADB_OPT_FLAG_ZERO_DATE_TO_MIN},
   {LASTPAGE, ckDebug,                  MADB_OPT_FLAG_DEBUG},
   {LASTPAGE, ckReturnMatchedRows,      MADB_OPT_FLAG_FOUND_ROWS},
+  {LASTPAGE, ckIgnoreSpace,            MADB_OPT_FLAG_IGNORE_SPACE},
   /* last element */
   {0, 0, 0}
 };
@@ -441,7 +443,7 @@ void MADB_WIN_TestDsn(my_bool ShowSuccess)
   {
     char Info[1024];
 	if (ret == SQL_SUCCESS)
-      my_snprintf(Info, 1024, "Connection successfully established\n\nServer information: %s", mysql_get_server_info(((MADB_Dbc *)Connection)->mariadb));
+      _snprintf(Info, 1024, "Connection successfully established\n\nServer information: %s", mysql_get_server_info(((MADB_Dbc *)Connection)->mariadb));
     MessageBox(hwndTab[CurrentPage],  (ret == SQL_SUCCESS) ? Info : 
 	         ((MADB_Dbc *)Connection)->Error.SqlErrorMsg, "Connection test",MB_ICONINFORMATION| MB_OK);
   }
