@@ -81,14 +81,14 @@ MADB_Desc *MADB_DescInit(MADB_Dbc *Dbc,enum enum_madb_desc_type DescType, my_boo
   Desc->DescType= DescType;
   MADB_PutErrorPrefix(Dbc, &Desc->Error);
 
-  if (my_init_dynamic_array(&Desc->Records, sizeof(MADB_DescRecord), 0, MADB_DESC_INIT_REC_NUM))
+  if (MADB_InitDynamicArray(&Desc->Records, sizeof(MADB_DescRecord), 0, MADB_DESC_INIT_REC_NUM))
   {
     MADB_FREE(Desc);
     return NULL;
   }
   if (isExternal)
   {
-    if (my_init_dynamic_array(&Desc->Stmts, sizeof(MADB_Stmt**), 0, MADB_DESC_INIT_STMT_NUM))
+    if (MADB_InitDynamicArray(&Desc->Stmts, sizeof(MADB_Stmt**), 0, MADB_DESC_INIT_STMT_NUM))
     {
       MADB_DescFree(Desc, FALSE);
       return NULL;
@@ -98,7 +98,7 @@ MADB_Desc *MADB_DescInit(MADB_Dbc *Dbc,enum enum_madb_desc_type DescType, my_boo
       Desc->Dbc= Dbc;
       /* MADB_DescInit call for explicit descriptor is in critical section */
       Desc->ListItem.data= (void *)Desc;
-      Dbc->Descrs= list_add(Dbc->Descrs, &Desc->ListItem);
+      Dbc->Descrs= MADB_ListAdd(Dbc->Descrs, &Desc->ListItem);
     }
   }
   Desc->AppType= isExternal;
@@ -139,7 +139,7 @@ SQLRETURN MADB_DescFree(MADB_Desc *Desc, my_bool RecordsOnly)
       MADB_FREE(Record->TypeName);
     }
   }
-  delete_dynamic(&Desc->Records);
+  MADB_DeleteDynamic(&Desc->Records);
 
   Desc->Header.Count= 0;
 
@@ -156,10 +156,10 @@ SQLRETURN MADB_DescFree(MADB_Desc *Desc, my_bool RecordsOnly)
       break;
     }
   }
-  delete_dynamic(&Desc->Stmts);
+  MADB_DeleteDynamic(&Desc->Stmts);
   if (Desc->AppType)
   {
-    Desc->Dbc->Descrs= list_delete(Desc->Dbc->Stmts, &Desc->ListItem);
+    Desc->Dbc->Descrs= MADB_ListDelete(Desc->Dbc->Stmts, &Desc->ListItem);
   }
   if (!RecordsOnly)
     MADB_FREE(Desc);
@@ -255,9 +255,9 @@ MADB_SetIrdRecord(MADB_Stmt *Stmt, MADB_DescRecord *Record, MYSQL_FIELD *Field)
                        Record->ConciseType == SQL_WLONGVARCHAR ||
                        Record->ConciseType == SQL_LONGVARBINARY) ? SQL_PRED_CHAR : SQL_SEARCHABLE;
 
-  Record->DisplaySize= MADB_GetDisplaySize(*Field, mysql_get_charset_by_nr(Field->charsetnr));
+  Record->DisplaySize= MADB_GetDisplaySize(*Field, mariadb_get_charset_by_nr(Field->charsetnr));
   Record->OctetLength= MADB_GetOctetLength(*Field, Stmt->Connection->mariadb->charset->char_maxlen);
-  Record->Length=      MADB_GetDataSize(Record, *Field, mysql_get_charset_by_nr(Field->charsetnr));
+  Record->Length= MADB_GetDataSize(Record, *Field, mariadb_get_charset_by_nr(Field->charsetnr));
     
   MADB_RESET(Record->TypeName, MADB_GetTypeName(*Field));
   switch(Field->type) {
@@ -342,13 +342,13 @@ void MADB_FixOctetLength(MADB_DescRecord *Record)
     Record->OctetLength= SQL_TIMESTAMP_LEN;
     break;
   default:
-    Record->OctetLength= MIN(INT_MAX32, Record->OctetLength);
+    Record->OctetLength= MIN(MADB_INT_MAX32, Record->OctetLength);
   }
 }
 /* }}} */
 
 /* {{{ MADB_FixDisplayLength */
-void MADB_FixDisplaySize(MADB_DescRecord *Record, const CHARSET_INFO *charset)
+void MADB_FixDisplaySize(MADB_DescRecord *Record, const MARIADB_CHARSET_INFO *charset)
 {
   switch (Record->ConciseType) {
   case SQL_BIT:
@@ -408,7 +408,7 @@ void MADB_FixDisplaySize(MADB_DescRecord *Record, const CHARSET_INFO *charset)
 /* }}} */
 
 /* {{{ MADB_FixDataSize - aka Column size */
-void MADB_FixDataSize(MADB_DescRecord *Record, const CHARSET_INFO *charset)
+void MADB_FixDataSize(MADB_DescRecord *Record, const MARIADB_CHARSET_INFO *charset)
 {
   switch (Record->ConciseType) {
   case SQL_BIT:
@@ -539,7 +539,7 @@ MADB_FixIrdRecord(MADB_Stmt *Stmt, MADB_DescRecord *Record)
   MADB_FixDisplaySize(Record, Stmt->Connection->mariadb->charset);
   MADB_FixDataSize(Record, Stmt->Connection->mariadb->charset);
     
-  /*Record->TypeName= my_strdup(MADB_GetTypeName(Fields[i]), MYF(0));*/
+  /*Record->TypeName= strdup(MADB_GetTypeName(Fields[i]));*/
 
   switch(Record->ConciseType) {
   case SQL_BINARY:
@@ -656,7 +656,7 @@ MADB_DescRecord *MADB_DescGetInternalRecord(MADB_Desc *Desc, SQLSMALLINT RecordN
 
   while (RecordNumber >= (SQLINTEGER)Desc->Records.elements)
   {
-    if (!(DescRecord= (MADB_DescRecord *)alloc_dynamic(&Desc->Records)))
+    if (!(DescRecord= (MADB_DescRecord *)MADB_AllocDynamic(&Desc->Records)))
     {
       MADB_SetError(&Desc->Error, MADB_ERR_HY001, NULL, 0);
       return NULL;
@@ -667,7 +667,9 @@ MADB_DescRecord *MADB_DescGetInternalRecord(MADB_Desc *Desc, SQLSMALLINT RecordN
 
   if (RecordNumber + 1 > Desc->Header.Count)
     Desc->Header.Count= (SQLSMALLINT)(RecordNumber + 1);
+
   DescRecord= ((MADB_DescRecord *)Desc->Records.buffer) + RecordNumber;
+
   return DescRecord;
 }
 /* }}} */
@@ -736,7 +738,7 @@ SQLRETURN MADB_DescGetField(SQLHDESC DescriptorHandle,
     *((SQLPOINTER *)ValuePtr)= (SQLPOINTER)Desc->Header.BindOffsetPtr;
     break;
   case SQL_DESC_BIND_TYPE:
-    *((SQLULEN *)ValuePtr)= Desc->Header.BindType;
+    *((SQLINTEGER *)ValuePtr)= Desc->Header.BindType;
     break;
   case SQL_DESC_COUNT:
     *(SQLSMALLINT *)ValuePtr= Desc->Header.Count;
@@ -973,9 +975,7 @@ SQLRETURN MADB_DescSetField(SQLHDESC DescriptorHandle,
       break;
     }
   
-    /* inUse is only used to check if column/parameter was bound or not. Thus we do not set it for each field, but only for those,
-       that make column/parameter "bound" */
-    if (DescRecord && (DescRecord->DataPtr != NULL || DescRecord->OctetLengthPtr != NULL || DescRecord->IndicatorPtr != NULL))
+    if (DescRecord)
       DescRecord->inUse= 1;
   }
   return ret;
@@ -999,8 +999,8 @@ SQLRETURN MADB_DescCopyDesc(MADB_Desc *SrcDesc, MADB_Desc *DestDesc)
     return SQL_ERROR;
   }
   /* make sure there aren't old records */
-  delete_dynamic(&DestDesc->Records);
-  if (my_init_dynamic_array(&DestDesc->Records, sizeof(MADB_DescRecord),
+  MADB_DeleteDynamic(&DestDesc->Records);
+  if (MADB_InitDynamicArray(&DestDesc->Records, sizeof(MADB_DescRecord),
                             SrcDesc->Records.max_element, SrcDesc->Records.alloc_increment))
   {
     MADB_SetError(&DestDesc->Error, MADB_ERR_HY001, NULL, 0);
