@@ -65,9 +65,8 @@ MADB_DsnKey DsnKeys[]=
   {"SSLCRL",         offsetof(MADB_Dsn, SslCrl),            DSN_TYPE_STRING, 0, 0},
   {"SSLCRLPATH",     offsetof(MADB_Dsn, SslCrlPath),        DSN_TYPE_STRING, 0, 0},
   {"SOCKET",         offsetof(MADB_Dsn, Socket),            DSN_TYPE_STRING, 0, 0},
-  {"SAVEFILE",       offsetof(MADB_Dsn, SaveFile),            DSN_TYPE_STRING, 0, 0}, /* 30 */
   /* Aliases. Here offset is index of aliased key */
-  {"SERVERNAME",     DSNKEY_SERVER_INDEX,                   DSN_TYPE_STRING, 0, 1},
+  {"SERVERNAME",     DSNKEY_SERVER_INDEX,                   DSN_TYPE_STRING, 0, 1}, /* 30 */
   {"USER",           DSNKEY_UID_INDEX,                      DSN_TYPE_STRING, 0, 1},
   {"PASSWORD",       DSNKEY_PWD_INDEX,                      DSN_TYPE_STRING, 0, 1},
   {"DB",             DSNKEY_DATABASE_INDEX,                 DSN_TYPE_COMBO,  0, 1},
@@ -137,7 +136,6 @@ void MADB_DSN_Free(MADB_Dsn *Dsn)
   MADB_FREE(Dsn->SslCrlPath);
   MADB_FREE(Dsn->SslFp);
   MADB_FREE(Dsn->SslFpList);
-  MADB_FREE(Dsn->SaveFile);
 
   if (Dsn->FreeMe)
     MADB_FREE(Dsn); 
@@ -375,7 +373,7 @@ my_bool MADB_SaveDSN(MADB_Dsn *Dsn)
         break;
       case DSN_TYPE_INT:
         {
-          my_snprintf(Value ,32, "%d", *(int *)((char *)Dsn + DsnKeys[i].DsnOffset));
+          _snprintf(Value ,32, "%d", *(int *)((char *)Dsn + DsnKeys[i].DsnOffset));
           ret= SQLWritePrivateProfileString(Dsn->DSNName, DsnKeys[i].DsnKey, Value, "ODBC.INI");
         }
         break;
@@ -397,7 +395,7 @@ my_bool MADB_SaveDSN(MADB_Dsn *Dsn)
     i++;
   }
   /* Save Options */
-  my_snprintf(Value ,32, "%d", Dsn->Options);
+  _snprintf(Value ,32, "%d", Dsn->Options);
   if (!(ret= SQLWritePrivateProfileString(Dsn->DSNName, "OPTIONS", Value, "ODBC.INI")))
   {
     SQLInstallerError(1,&ErrNum, Dsn->ErrorMsg, SQL_MAX_MESSAGE_LENGTH, NULL);
@@ -509,15 +507,15 @@ BOOL MADB_ReadConnString(MADB_Dsn *Dsn, const char *String, size_t Length, char
 /* }}} */
 
 /* {{{ MADB_DsnToString */
-SQLULEN MADB_DsnToString(MADB_Dsn *Dsn, char *OutString, SQLULEN OutLength)
+SQLSMALLINT MADB_DsnToString(MADB_Dsn *Dsn, char *OutString, SQLSMALLINT OutLength)
 {
-  int     i=           0;
-  SQLULEN TotalLength= 0;
-  char    *p=          OutString;
-  char    *Value=      NULL;
-  char    TmpStr[1024];
-  char    IntVal[12];
-  int     CpyLength;
+  int           i=           0;
+  SQLSMALLINT   TotalLength= 0;
+  char          *p=          OutString;
+  char          *Value=      NULL;
+  char          TmpStr[1024];
+  char          IntVal[12];
+  int           CpyLength;
 
   if (OutLength && OutString)
     OutString[0]= '\0';
@@ -526,17 +524,11 @@ SQLULEN MADB_DsnToString(MADB_Dsn *Dsn, char *OutString, SQLULEN OutLength)
   {
     Value= NULL;
 
-    if (!DsnKeys[i].IsAlias)
-    {
+    if (!DsnKeys[i].IsAlias) {
       switch (DsnKeys[i].Type) {
       case DSN_TYPE_STRING:
       case DSN_TYPE_COMBO:
          Value= *GET_FIELD_PTR(Dsn, &DsnKeys[i], char*);
-         if (MADB_IS_EMPTY(Value))
-         {
-           ++i;
-           continue;
-         }
          break;
       case DSN_TYPE_INT:
         if (*GET_FIELD_PTR(Dsn, &DsnKeys[i], int))
@@ -557,7 +549,7 @@ SQLULEN MADB_DsnToString(MADB_Dsn *Dsn, char *OutString, SQLULEN OutLength)
     if (Value)
     {
       my_bool isSpecial= (strchr(Value, ' ') ||  strchr(Value, ';') || strchr(Value, '@'));
-      CpyLength= my_snprintf(TmpStr + TotalLength, 1024 - TotalLength, "%s%s=%s%s%s", (TotalLength) ? ";" : "",
+      CpyLength= _snprintf(TmpStr + TotalLength, 1024 - TotalLength, "%s%s=%s%s%s", (TotalLength) ? ";" : "",
                              DsnKeys[i].DsnKey, isSpecial ? "{" : "", Value, isSpecial ? "}" : "");
       TotalLength+= CpyLength;
     }
