@@ -1,5 +1,5 @@
 /************************************************************************************
-   Copyright (C) 2013 SkySQL AB
+   Copyright (C) 2013,2016 MariaDB Corporation AB
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -16,7 +16,7 @@
    or write to the Free Software Foundation, Inc., 
    51 Franklin St., Fifth Floor, Boston, MA 02110, USA
 *************************************************************************************/
-#ifndef _ma_dsn_h_
+#ifndef _na_dsn_h_
 #define _ma_dsn_h_
 
 #include <odbcinst.h>
@@ -120,7 +120,6 @@ typedef struct st_madb_dsn
   char *SslFp;
   char *SslFpList;
   my_bool SslVerify;
-  char *SaveFile;
   /* --- Internal --- */
   int isPrompt;
   MADB_DsnKey *Keys;
@@ -144,7 +143,7 @@ my_bool     MADB_SaveDSN        (MADB_Dsn *Dsn);
 my_bool     MADB_DSN_Exists     (const char *DsnName);
 my_bool     MADB_ParseConnString(MADB_Dsn *Dsn, const char *String, size_t Length, char Delimiter);
 BOOL        MADB_ReadConnString (MADB_Dsn *Dsn, const char *String, size_t Length, char Delimiter);
-SQLULEN     MADB_DsnToString    (MADB_Dsn *Dsn, char *OutString, SQLULEN OutLength);
+SQLSMALLINT MADB_DsnToString    (MADB_Dsn *Dsn, char *OutString, SQLSMALLINT OutLength);
 void        MADB_DsnUpdateOptionsFields(MADB_Dsn *Dsn);
 
 /*** Helper macros ***/
@@ -163,8 +162,12 @@ void        MADB_DsnUpdateOptionsFields(MADB_Dsn *Dsn);
     if ((len) == SQL_NTS)\
       (len)=(SQLSMALLINT)strlen((value));\
     MADB_FREE((dsn)->item);\
-    (dsn)->item= (char *)my_malloc(len + 1, MYF(MY_ZEROFILL));\
+    (dsn)->item= (char *)calloc(len + 1, sizeof(char));\
     memcpy((dsn)->item, (value),(len));\
   }
 
+#define MADB_DSN_GET_UTF8(dsn, item, len) \
+  ((dsn)->item) ? MADB_ConvertFromWChar((dsn)->item, wcslen((dsn)->item)+1, (len)) : NULL
+
+
 #endif /* _ma_dsn_h_ */
