@@ -25,100 +25,185 @@ void CloseMultiStatements(MADB_Stmt *Stmt)
 {
   unsigned int i;
 
-  for (i=0; i < Stmt->Query.MultiStmtCount; ++i)
+  for (i=0; i < Stmt->MultiStmtCount; ++i)
   {
     MDBUG_C_PRINT(Stmt->Connection, "-->closing %0x", Stmt->MultiStmts[i]);
-    if (Stmt->MultiStmts[i] != NULL)
-    {
-      mysql_stmt_close(Stmt->MultiStmts[i]);
-    }
+    mysql_stmt_close(Stmt->MultiStmts[i]);
   }
   MADB_FREE(Stmt->MultiStmts);
-  Stmt->Query.MultiStmtCount= 0;
+  Stmt->MultiStmtCount= 0;
   Stmt->stmt= NULL;
 }
 
 
 /* Required, but not sufficient condition */
-BOOL QueryIsPossiblyMultistmt(MADB_QUERY *Query)
+BOOL QueryIsPossiblyMultistmt(char *queryStr)
 {
-  return Query->QueryType != MADB_QUERY_CREATE_PROC && Query->QueryType != MADB_QUERY_CREATE_FUNC &&
-         Query->QueryType != MADB_QUERY_CREATE_DEFINER;
+  char *semicolon_pos= strchr(queryStr, ';');
+  /* String supposed to come here trimmed. Checking that semicolon is not last char in the string */
+  if (semicolon_pos != NULL && semicolon_pos < queryStr + strlen(queryStr) - 1)
+  {
+    /* CREATE PROCEDURE uses semicolons but is not supported in prepared statement
+        protocol */
+    if (!MADB_IsStatementSupported(queryStr, "CREATE", "PROCEDURE"))
+      return FALSE;
+    if (!MADB_IsStatementSupported(queryStr, "CREATE", "FUNCTION"))
+      return FALSE;
+    if (!MADB_IsStatementSupported(queryStr, "CREATE", "DEFINER"))
+      return FALSE;
+    return TRUE;
+  }
+
+  return FALSE;
 }
 
-
-/* Trims spaces and/or ';' at the end of query */
-int SqlRtrim(char *StmtStr, int Length)
+#define STRING_OR_COMMENT() (quote[0] || quote[1] || comment)
+unsigned int GetMultiStatements(MADB_Stmt *Stmt, char *StmtStr, SQLINTEGER Length)
 {
-  if (Length > 0)
+  char *p, *prev= NULL, wait_char= 0;
+  unsigned int statements= 1;
+  int quote[2]= {0,0}, comment= 0;
+  char *end;
+  MYSQL_STMT *stmt;
+  char *StmtCopy= NULL;
+
+  /* mysql_stmt_init requires locking as well */
+  LOCK_MARIADB(Stmt->Connection);
+  stmt= mysql_stmt_init(Stmt->Connection->mariadb);
+
+  /* if the entire stmt string passes, we don't have multistatement */
+  if (stmt && !mysql_stmt_prepare(stmt, StmtStr, Length))
   {
-    char *end= StmtStr + Length - 1;
-    while (end > StmtStr && (isspace(0x000000ff & *end) || *end == ';'))
+    mysql_stmt_close(stmt);
+    UNLOCK_MARIADB(Stmt->Connection);
+    return 1;
+  }
+  mysql_stmt_close(stmt);
+
+  /* make sure we don't have trailing whitespace or semicolon */
+  if (Length)
+  {
+    end= StmtStr + Length - 1;
+    while (end > StmtStr && (isspace(*end) || *end == ';'))
     {
-      *end= '\0';
       --end;
       --Length;
     }
   }
+  /* We can have here not NULL-terminated string as a source, thus we need to allocate, copy meaningful characters and
+     add NULL */
+  p= StmtCopy= MADB_ALLOC(Length + 1);
+  strncpy(StmtCopy, StmtStr, Length);
+  StmtCopy[Length]= '\0';
 
-  return Length;
-}
-
-
-unsigned int GetMultiStatements(MADB_Stmt *Stmt)
-{
-  int          i= 0;
-  unsigned int MaxParams= 0;
-  char        *p= Stmt->Query.RefinedText;
-
-  Stmt->MultiStmtNr= 0;
-  Stmt->MultiStmts= (MYSQL_STMT **)MADB_CALLOC(sizeof(MYSQL_STMT) * Stmt->Query.MultiStmtCount);
-
-  while (p < Stmt->Query.RefinedText + Stmt->Query.RefinedLength)
+  end= StmtCopy + Length;
+  
+  while (p < end)
   {
-    Stmt->MultiStmts[i]= i == 0 ? Stmt->stmt : mysql_stmt_init(Stmt->Connection->mariadb);
-    MDBUG_C_PRINT(Stmt->Connection, "-->inited&preparing %0x(%d,%s)", Stmt->MultiStmts[i], i, p);
-
-    if (mysql_stmt_prepare(Stmt->MultiStmts[i], p, (unsigned long)strlen(p)))
+    if (wait_char)
     {
-      MADB_SetNativeError(&Stmt->Error, SQL_HANDLE_STMT, Stmt->MultiStmts[i]);
-      CloseMultiStatements(Stmt);
-
-      /* Last paranoid attempt make sure that we did not have a parsing error.
-         More to preserve "backward-compatimility" - we did this before, but before trying to
-         prepare "multi-statement". */
-      if (i == 0 && Stmt->Error.NativeError !=1295 /*ER_UNSUPPORTED_PS*/)
+      if (*p == wait_char && *prev != '\\')
       {
-        Stmt->stmt= mysql_stmt_init(Stmt->Connection->mariadb);
-        if (mysql_stmt_prepare(Stmt->stmt, STMT_STRING(Stmt), (unsigned long)strlen(STMT_STRING(Stmt))))
-        {
-          mysql_stmt_close(Stmt->stmt);
-          Stmt->stmt= NULL;
-        }
-        else
-        {
-          Stmt->Query.MultiStmtCount= 1;
-          return 0;
-        }
+        wait_char= 0;
       }
-      return 1;
     }
-    if (mysql_stmt_param_count(Stmt->MultiStmts[i]) > MaxParams)
+    else
     {
-      MaxParams= mysql_stmt_param_count(Stmt->MultiStmts[i]);
+      switch (*p) {
+      case '-':
+        if (!STRING_OR_COMMENT() && (p < end - 1) && (char)*(p+1) ==  '-')
+        {
+          wait_char= '\n';
+        }
+        break;
+      case '#':
+        if (!STRING_OR_COMMENT())
+        {
+          wait_char= '\n';
+        }
+        break;
+      case ';':
+        if (!STRING_OR_COMMENT())
+        {
+          statements++;
+          *p= '\0';
+        }
+        break;
+      case '/':
+        if (!STRING_OR_COMMENT() && (p < end - 1) && (char)*(p+1) ==  '*')
+          comment= 1;
+        else if (comment && (p > StmtCopy) && (char)*(prev) == '*')
+          comment= 0;
+        break;
+      case '"':
+        quote[0] = !STRING_OR_COMMENT();
+        break;
+      case '\'':
+        quote[1] = !STRING_OR_COMMENT();
+        break;
+      case '\\':
+        /* *end is \0, so we are safe here to increment by 2 bytes */
+        if ((Stmt->Connection->mariadb->server_status & SERVER_STATUS_NO_BACKSLASH_ESCAPES) == 0  && p < end - 1)
+        {
+          /* Skipping escaped character and resetting prev */
+          p+= 2;
+          prev= 0;
+          continue;
+        }
+      default:
+        break;
+      }
     }
-    p+= strlen(p) + 1;
-    ++i;
+
+    prev= p;
+    ++p;
   }
 
-  if (MaxParams)
+  if (statements > 1)
   {
-    Stmt->params= (MYSQL_BIND *)MADB_CALLOC(sizeof(MYSQL_BIND) * MaxParams);
+    int i=0;
+    unsigned int MaxParams= 0;
+
+    p= StmtCopy;
+    Stmt->MultiStmtCount= 0;
+    Stmt->MultiStmtNr= 0;
+    Stmt->MultiStmts= (MYSQL_STMT **)MADB_CALLOC(sizeof(MYSQL_STMT) * statements);
+
+    while (p < StmtCopy + Length)
+    {
+      /* Need to be incremented before CloseMultiStatements() */
+      ++Stmt->MultiStmtCount;
+      Stmt->MultiStmts[i]= i == 0 ? Stmt->stmt : mysql_stmt_init(Stmt->Connection->mariadb);
+      MDBUG_C_PRINT(Stmt->Connection, "-->inited&preparing %0x(%d,%s)", Stmt->MultiStmts[i], i, p);
+
+      if (mysql_stmt_prepare(Stmt->MultiStmts[i], p, (unsigned long)strlen(p)))
+      {
+        MADB_SetNativeError(&Stmt->Error, SQL_HANDLE_STMT, Stmt->MultiStmts[i]);
+        CloseMultiStatements(Stmt);
+        if (StmtCopy)
+        {
+          free(StmtCopy);
+        }
+        UNLOCK_MARIADB(Stmt->Connection);
+
+        return 0;
+      }
+      if (mysql_stmt_param_count(Stmt->MultiStmts[i]) > MaxParams)
+        MaxParams= mysql_stmt_param_count(Stmt->MultiStmts[i]);
+      p+= strlen(p) + 1;
+      ++i;
+    }
+    UNLOCK_MARIADB(Stmt->Connection);
+
+    if (MaxParams)
+      Stmt->params= (MYSQL_BIND *)MADB_CALLOC(sizeof(MYSQL_BIND) * MaxParams);
   }
+  if (StmtCopy)
+    free(StmtCopy);
 
-  return 0;
+  return statements;
 }
-
+#undef STRING_OR_COMMENT
 
 my_bool MADB_CheckPtrLength(SQLINTEGER MaxLength, char *Ptr, SQLINTEGER NameLen)
 {
@@ -154,15 +239,18 @@ int MADB_KeyTypeCount(MADB_Dbc *Connection, char *TableName, int KeyFlag)
   MADB_Stmt    *KeyStmt;
   
   Connection->Methods->GetAttr(Connection, SQL_ATTR_CURRENT_CATALOG, Database, 65, NULL, FALSE);
-  p+= my_snprintf(p, 1024, "SELECT * FROM ");
+  p+= _snprintf(p, 1024, "SELECT * FROM ");
   if (Database)
-    p+= my_snprintf(p, 1024 - strlen(p), "`%s`.", Database);
-  p+= my_snprintf(p, 1024 - strlen(p), "%s LIMIT 0", TableName);
+  {
+    p+= _snprintf(p, 1024 - strlen(p), "`%s`.", Database);
+  }
+  p+= _snprintf(p, 1024 - strlen(p), "%s LIMIT 0", TableName);
   if (MA_SQLAllocHandle(SQL_HANDLE_STMT, (SQLHANDLE)Connection, (SQLHANDLE*)&Stmt) == SQL_ERROR ||
-      Stmt->Methods->Prepare(Stmt, (SQLCHAR *)StmtStr, SQL_NTS) == SQL_ERROR ||
-      Stmt->Methods->Execute(Stmt) == SQL_ERROR ||
-      Stmt->Methods->Fetch(Stmt) == SQL_ERROR)
-      goto end;
+    Stmt->Methods->ExecDirect(Stmt, (SQLCHAR *)StmtStr, SQL_NTS) == SQL_ERROR ||
+    Stmt->Methods->Fetch(Stmt) == SQL_ERROR)
+  {
+    goto end;
+  }
   KeyStmt= (MADB_Stmt *)Stmt;
   for (i=0; i < mysql_stmt_field_count(KeyStmt->stmt); i++)
     if (KeyStmt->stmt->fields[i].flags & KeyFlag)
@@ -208,7 +296,7 @@ my_bool MADB_get_single_row(MADB_Dbc *Connection,
 /* }}} */
 
 /* {{{ MADB_CheckODBCType */
-bool MADB_CheckODBCType(SQLSMALLINT Type)
+BOOL MADB_CheckODBCType(SQLSMALLINT Type)
 {
   switch(Type)
   {
@@ -337,15 +425,15 @@ char *MADB_GetTypeName(MYSQL_FIELD Field)
 
 MYSQL_RES *MADB_GetDefaultColumnValues(MADB_Stmt *Stmt, MYSQL_FIELD *fields)
 {
-  DYNAMIC_STRING DynStr;
+  MADB_DynString DynStr;
   unsigned int i;
   MYSQL_RES *result= NULL;
   
-  init_dynamic_string(&DynStr, "SELECT COLUMN_NAME, COLUMN_DEFAULT FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA='", 512, 512);
-  if (dynstr_append(&DynStr, fields[0].db) ||
-      dynstr_append(&DynStr, "' AND TABLE_NAME='") ||
-      dynstr_append(&DynStr, fields[0].org_table) ||
-      dynstr_append(&DynStr, "' AND COLUMN_NAME IN ("))
+  MADB_InitDynamicString(&DynStr, "SELECT COLUMN_NAME, COLUMN_DEFAULT FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA='", 512, 512);
+  if (MADB_DynstrAppend(&DynStr, fields[0].db) ||
+      MADB_DynstrAppend(&DynStr, "' AND TABLE_NAME='") ||
+      MADB_DynstrAppend(&DynStr, fields[0].org_table) ||
+      MADB_DynstrAppend(&DynStr, "' AND COLUMN_NAME IN ("))
     goto error;
 
   for (i=0; i < mysql_stmt_field_count(Stmt->stmt); i++)
@@ -356,12 +444,14 @@ MYSQL_RES *MADB_GetDefaultColumnValues(MADB_Stmt *Stmt, MYSQL_FIELD *fields)
     {
       continue;
     }
-    if (dynstr_append(&DynStr, i > 0 ? ",'" : "'") ||
-        dynstr_append(&DynStr, fields[i].org_name) ||
-        dynstr_append(&DynStr, "'"))
+    if (MADB_DynstrAppend(&DynStr, i > 0 ? ",'" : "'") ||
+      MADB_DynstrAppend(&DynStr, fields[i].org_name) ||
+      MADB_DynstrAppend(&DynStr, "'"))
+    {
       goto error;
+    }
   }
-  if (dynstr_append(&DynStr, ") AND COLUMN_DEFAULT IS NOT NULL"))
+  if (MADB_DynstrAppend(&DynStr, ") AND COLUMN_DEFAULT IS NOT NULL"))
     goto error;
 
   LOCK_MARIADB(Stmt->Connection);
@@ -371,7 +461,7 @@ MYSQL_RES *MADB_GetDefaultColumnValues(MADB_Stmt *Stmt, MYSQL_FIELD *fields)
   
 error:
     UNLOCK_MARIADB(Stmt->Connection);
-    dynstr_free(&DynStr);
+    MADB_DynstrFree(&DynStr);
     return result;
 }
 
@@ -390,7 +480,7 @@ char *MADB_GetDefaultColumnValue(MYSQL_RES *res, const char *Column)
   return NULL;
 }
 
-size_t MADB_GetDataSize(MADB_DescRecord *Record, MYSQL_FIELD Field, CHARSET_INFO *charset)
+size_t MADB_GetDataSize(MADB_DescRecord *Record, MYSQL_FIELD Field, MARIADB_CHARSET_INFO *charset)
 {
   switch(Record->ConciseType)
   {
@@ -433,7 +523,7 @@ size_t MADB_GetDataSize(MADB_DescRecord *Record, MYSQL_FIELD Field, CHARSET_INFO
 }
 
 /* {{{ MADB_GetDisplaySize */
-size_t MADB_GetDisplaySize(MYSQL_FIELD Field, CHARSET_INFO *charset)
+size_t MADB_GetDisplaySize(MYSQL_FIELD Field, MARIADB_CHARSET_INFO *charset)
 {
   /* Todo: check these values with output from mysql --with-columntype-info */
   switch (Field.type) {
@@ -500,7 +590,7 @@ size_t MADB_GetDisplaySize(MYSQL_FIELD Field, CHARSET_INFO *charset)
 /* {{{ MADB_GetOctetLength */
 size_t MADB_GetOctetLength(MYSQL_FIELD Field, unsigned short MaxCharLen)
 {
-  size_t Length= MIN(INT_MAX32,Field.length);
+  size_t Length= MIN(MADB_INT_MAX32, Field.length);
 
   switch (Field.type) {
   case MYSQL_TYPE_NULL:
@@ -681,7 +771,7 @@ size_t MADB_GetTypeLength(SQLINTEGER SqlDataType, size_t Length)
     return sizeof(SQLINTEGER);
   case SQL_C_UBIGINT:
   case SQL_C_SBIGINT:
-    return sizeof(longlong);
+    return sizeof(long long);
   case SQL_C_DOUBLE:
     return sizeof(SQLDOUBLE);
   case SQL_C_FLOAT:
@@ -703,8 +793,8 @@ size_t MADB_GetTypeLength(SQLINTEGER SqlDataType, size_t Length)
 }
 /* }}} */
 
-/* {{{ MADB_GetTypeAndLength */
-int MADB_GetTypeAndLength(SQLINTEGER SqlDataType, my_bool *Unsigned, unsigned long *Length)
+/* {{{ MADB_GetMaDBTypeAndLength */
+int MADB_GetMaDBTypeAndLength(SQLINTEGER SqlDataType, my_bool *Unsigned, unsigned long *Length)
 {
   *Unsigned= 0;
   switch(SqlDataType)
@@ -734,7 +824,7 @@ int MADB_GetTypeAndLength(SQLINTEGER SqlDataType, my_bool *Unsigned, unsigned lo
     return MYSQL_TYPE_LONG;
   case SQL_C_UBIGINT:
   case SQL_C_SBIGINT:
-    *Length= sizeof(longlong);
+    *Length= sizeof(long long);
     *Unsigned= (SqlDataType == SQL_C_UBIGINT);
     return MYSQL_TYPE_LONGLONG;
   case SQL_C_DOUBLE:
@@ -744,7 +834,7 @@ int MADB_GetTypeAndLength(SQLINTEGER SqlDataType, my_bool *Unsigned, unsigned lo
     *Length =sizeof(SQLFLOAT);
     return MYSQL_TYPE_FLOAT;
   case SQL_C_NUMERIC:
-    *Length= sizeof(SQL_NUMERIC_STRUCT);
+    /**Length= sizeof(SQL_NUMERIC_STRUCT);*/
     return MYSQL_TYPE_DECIMAL;
   case SQL_C_TYPE_TIME:
   case SQL_C_TIME:
@@ -758,6 +848,10 @@ int MADB_GetTypeAndLength(SQLINTEGER SqlDataType, my_bool *Unsigned, unsigned lo
   case SQL_C_TIMESTAMP:
     *Length= sizeof(SQL_TIMESTAMP_STRUCT);
     return MYSQL_TYPE_TIMESTAMP;
+  case SQL_C_INTERVAL_HOUR_TO_MINUTE:
+  case SQL_C_INTERVAL_HOUR_TO_SECOND:
+    *Length= sizeof(SQL_INTERVAL_STRUCT);
+    return MYSQL_TYPE_TIME;
   case SQL_C_CHAR:
     return MYSQL_TYPE_STRING;
   default:
@@ -879,11 +973,10 @@ void MADB_NumericInit(SQL_NUMERIC_STRUCT *number, MADB_DescRecord *Ard)
 }
 
 /* {{{ MADB_CharToSQLNumeric */
-int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdRecord, SQL_NUMERIC_STRUCT *dst_buffer, unsigned long RowNumber)
+int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdRecord, unsigned long RowNumber)
 {
   char *p;
-  SQL_NUMERIC_STRUCT *number= dst_buffer != NULL ? dst_buffer :
-    (SQL_NUMERIC_STRUCT *)GetBindOffset(Ard, ArdRecord, ArdRecord->DataPtr, RowNumber, ArdRecord->OctetLength);
+  SQL_NUMERIC_STRUCT *number= (SQL_NUMERIC_STRUCT *)GetBindOffset(Ard, ArdRecord, ArdRecord->DataPtr, RowNumber, ArdRecord->OctetLength);
   int ret= 0;
 
   if (!buffer || !number)
@@ -901,11 +994,9 @@ int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdReco
   {
     number->precision= MADB_DEFAULT_PRECISION;
   }
-
+   
   while (*p=='0')
-  {
     p++;
-  }
   if (*p)
   {
     int i;
@@ -930,18 +1021,17 @@ int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdReco
       while (*p)
       {
         /* ignore non numbers */
-        if (!isdigit(0x000000ff & *p))
+        if (!isdigit(*p))
           break;
         digits_total++;
         /* ignore trailing zeros */
         if (*p != '0')
-        {
           digits_significant= digits_total;
-        }
         p++;
       }
-
-      if (digits_count + number->scale > number->precision)
+      /* check possible overflow */
+      digits_significant= MIN(digits_significant,number->scale);
+      if (digits_count + digits_significant > number->precision)
       {
         int i;
         /* if digits are zero there is no overflow */
@@ -954,20 +1044,12 @@ int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdReco
       }
       
       memcpy(digits + digits_count, dot + 1, digits_significant);
-      if (number->scale > digits_significant)
-      {
-        for (i= digits_count + digits_significant; i < number->precision && i < digits_count +number->scale; ++i)
-        {
-          digits[i]= '0';
-        }
-        digits_significant= number->scale;
-      }
       digits_count+= digits_significant;
-    }
-    else 
+
+    } else 
     {
       char *start= p;
-      while (*p && isdigit(0x000000ff & *p))
+      while (*p && isdigit(*p))
         p++;
       /* check overflow */
       if (p - start > number->precision)
@@ -978,8 +1060,8 @@ int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdReco
       memcpy(digits, start, digits_count);
       number->scale= ArdRecord->Scale ? ArdRecord->Scale : 0;
     }
-
     /* Rounding */
+    
     if (number->scale < 0)
     {
       int64_t OldVal, Val;
@@ -997,8 +1079,7 @@ int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdReco
       if (digits_count > number->precision)
         return MADB_ERR_22003;
     }
-
-    digits_count= MIN(digits_count, MADB_DEFAULT_PRECISION);
+    digits_count= MIN(digits_count, 38);
     for (hval = 0, bit = 1L, sta = 0, olen = 0; sta < digits_count;)
     {
       for (dig = 0, i = sta; i < digits_count; i++)
@@ -1026,92 +1107,11 @@ int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdReco
       } 
     }
     if (hval && olen < SQL_MAX_NUMERIC_LEN - 1)
-    {
       number->val[olen++] = hval;
-    }
   } 
   return ret;
 }
 
-/* {{{ MADB_SqlNumericToChar */
-size_t MADB_SqlNumericToChar(SQL_NUMERIC_STRUCT *Numeric, char *Buffer, int *ErrorCode)
-{
-  longlong Numerator= 0;
-  longlong Denominator= 1;
-  ulonglong Left= 0, Right= 0;
-  int Scale= 0;
-  int ppos= 0;
-  long ByteDenominator= 1;
-  int i;
-  char *p;
-  my_bool hasDot= FALSE;
-
-  Buffer[0]= 0;
-  *ErrorCode= 0;
-
-  Scale+= (Numeric->scale < 0) ? -Numeric->scale : Numeric->scale;
-      
-  for (i=0; i < SQL_MAX_NUMERIC_LEN; ++i)
-  {
-    Numerator+= Numeric->val[i] * ByteDenominator;
-    ByteDenominator<<= 8;
-  }
-  if (!Numeric->sign)
-    Numerator= -Numerator;
-  Denominator= (longlong)pow(10, Scale);
-  Left= Numerator / Denominator;
-  //_i64toa_s(Numerator, Buffer, 38, 10);
-  if (Numeric->scale > 0)
-  {
-    char tmp[38];
-    _snprintf(tmp,38, "%%.%df", Numeric->scale);
-    _snprintf(Buffer, 38, tmp, Numerator / pow(10, Scale));
-  }
-  else
-  {
-    _snprintf(Buffer, 38, "%lld", Numerator);
-    while (strlen(Buffer) < (size_t)(Numeric->precision - Numeric->scale))
-      strcat(Buffer, "0");
-  }
-  
-  
-  if (Buffer[0] == '-')
-    Buffer++;
- 
- /* Truncation checks:
-     1st ensure, that the digits before decimal point will fit */
-  if ((p= strchr(Buffer, '.')))
-  {
-    if (p - Buffer - 1 > Numeric->precision)
-    {
-      *ErrorCode= MADB_ERR_22003;
-      Buffer[Numeric->precision]= 0;
-      goto end;
-    }
-    if (Numeric->scale > 0 && Left > 0 && (p - Buffer) + strlen(p) > Numeric->precision)
-    {
-      *ErrorCode= MADB_ERR_01S07;
-      Buffer[Numeric->precision + 1]= 0;
-      goto end;
-    }
-  }
-  while (Numeric->scale < 0 && strlen(Buffer) < (size_t)(Numeric->precision - Numeric->scale))
-    strcat(Buffer, "0");
-    
-
-  if (strlen(Buffer) > (size_t)(Numeric->precision + Scale) && Numeric->scale > 0)
-    *ErrorCode= MADB_ERR_01S07;
-
-end:
-    /* check if last char is decimal point */
-  if (strlen(Buffer) && Buffer[strlen(Buffer)-1] == '.')
-    Buffer[strlen(Buffer)-1] = 0;
-  if (!Numeric->sign)
-    Buffer--;
-  return strlen(Buffer);
-}
-/* }}} */
-
 /* {{{ MADB_GetHexString */
 size_t MADB_GetHexString(char *BinaryBuffer, size_t BinaryLength,
                           char *HexBuffer, size_t HexLength)
@@ -1152,10 +1152,10 @@ SQLRETURN MADB_DaeStmt(MADB_Stmt *Stmt, SQLUSMALLINT Operation)
 {
   char          *TableName=   MADB_GetTableName(Stmt);
   char          *CatalogName= MADB_GetCatalogName(Stmt);
-  DYNAMIC_STRING DynStmt;
+  MADB_DynString DynStmt;
 
   MADB_CLEAR_ERROR(&Stmt->Error);
-  memset(&DynStmt, 0, sizeof(DYNAMIC_STRING));
+  memset(&DynStmt, 0, sizeof(MADB_DynString));
 
   if (Stmt->DaeStmt)
     Stmt->Methods->StmtFree(Stmt->DaeStmt, SQL_DROP);
@@ -1170,58 +1170,82 @@ SQLRETURN MADB_DaeStmt(MADB_Stmt *Stmt, SQLUSMALLINT Operation)
   switch(Operation)
   {
   case SQL_ADD:
-    if (init_dynamic_string(&DynStmt, "INSERT INTO ", 1024, 1024) ||
+    if (MADB_InitDynamicString(&DynStmt, "INSERT INTO ", 1024, 1024) ||
         MADB_DynStrAppendQuoted(&DynStmt, CatalogName) ||
-        dynstr_append(&DynStmt, ".") ||
+        MADB_DynstrAppend(&DynStmt, ".") ||
         MADB_DynStrAppendQuoted(&DynStmt, TableName)||
         MADB_DynStrUpdateSet(Stmt, &DynStmt))
     {
-      dynstr_free(&DynStmt);
+      MADB_DynstrFree(&DynStmt);
       return Stmt->Error.ReturnValue;
     }
     Stmt->DataExecutionType= MADB_DAE_ADD;
     break;
   case SQL_DELETE:
-    if (init_dynamic_string(&DynStmt, "DELETE FROM ", 1024, 1024) ||
+    if (MADB_InitDynamicString(&DynStmt, "DELETE FROM ", 1024, 1024) ||
         MADB_DynStrAppendQuoted(&DynStmt, CatalogName) ||
-        dynstr_append(&DynStmt, ".") ||
+        MADB_DynstrAppend(&DynStmt, ".") ||
         MADB_DynStrAppendQuoted(&DynStmt, TableName) ||
         MADB_DynStrGetWhere(Stmt, &DynStmt, TableName, FALSE))
     {
-      dynstr_free(&DynStmt);
+      MADB_DynstrFree(&DynStmt);
       return Stmt->Error.ReturnValue;
     }
     Stmt->DataExecutionType= MADB_DAE_DELETE;
     break;
   case SQL_UPDATE:
-    Stmt->Methods->RefreshRowPtrs(Stmt);
-    if (init_dynamic_string(&DynStmt, "UPDATE ", 1024, 1024) ||
+    if (MADB_InitDynamicString(&DynStmt, "UPDATE ", 1024, 1024) ||
         MADB_DynStrAppendQuoted(&DynStmt, CatalogName) ||
-        dynstr_append(&DynStmt, ".") ||
+        MADB_DynstrAppend(&DynStmt, ".") ||
         MADB_DynStrAppendQuoted(&DynStmt, TableName)||
         MADB_DynStrUpdateSet(Stmt, &DynStmt)||
         MADB_DynStrGetWhere(Stmt, &DynStmt, TableName, FALSE))
     {
-      dynstr_free(&DynStmt);
+      MADB_DynstrFree(&DynStmt);
       return Stmt->Error.ReturnValue;
     }
     Stmt->DataExecutionType= MADB_DAE_UPDATE;
     break;
   }
   
-  if (!SQL_SUCCEEDED(Stmt->DaeStmt->Methods->Prepare(Stmt->DaeStmt, (SQLCHAR *)DynStmt.str, SQL_NTS)))
+  if (!SQL_SUCCEEDED(Stmt->DaeStmt->Methods->Prepare(Stmt->DaeStmt, (SQLCHAR *)DynStmt.str, SQL_NTS, FALSE)))
   {
     MADB_CopyError(&Stmt->Error, &Stmt->DaeStmt->Error);
     Stmt->Methods->StmtFree(Stmt->DaeStmt, SQL_DROP);
   }
    
 end:
-  dynstr_free(&DynStmt);
+  MADB_DynstrFree(&DynStmt);
   return Stmt->Error.ReturnValue;
 
 }
 
 
+int MADB_FindNextDaeParam(MADB_Desc *Desc, int InitialParam, SQLSMALLINT RowNumber)
+{
+  int             i;
+  MADB_DescRecord *Record;
+
+  for (i= InitialParam > -1 ? InitialParam + 1 : 0; i < Desc->Header.Count; i++)
+  {
+    if ((Record= MADB_DescGetInternalRecord(Desc, i, MADB_DESC_READ)))
+    {
+      if (Record->OctetLengthPtr)
+      {
+        /* Stmt->DaeRowNumber is 1 based */
+        SQLLEN *OctetLength = (SQLLEN *)GetBindOffset(Desc, Record, Record->OctetLengthPtr, RowNumber > 1 ? RowNumber - 1 : 0, sizeof(SQLLEN));
+        if (PARAM_IS_DAE(OctetLength))
+        {
+          return i;
+        }
+      }
+    }
+  }
+
+  return MADB_NOPARAM;
+}
+
+
 BOOL MADB_IsNumericType(SQLSMALLINT ConciseType)
 {
   switch (ConciseType)
@@ -1268,18 +1292,3 @@ BOOL MADB_IsIntType(SQLSMALLINT ConciseType)
   return FALSE;
 }
 
-
-void MADB_InstallStmt(MADB_Stmt *Stmt)
-{
-  Stmt->stmt= Stmt->MultiStmts[Stmt->MultiStmtNr];
-
-  if (mysql_stmt_field_count(Stmt->stmt) == 0)
-  {
-    Stmt->AffectedRows= mysql_stmt_affected_rows(Stmt->stmt);
-  }
-  else
-  {
-    MADB_StmtResetResultStructures(Stmt);
-    MADB_DescSetIrdMetadata(Stmt, mysql_fetch_fields(FetchMetadata(Stmt)), mysql_stmt_field_count(Stmt->stmt));
-  }
-}
