@@ -158,7 +158,7 @@ SQLRETURN MADB_GetTypeInfo(SQLHSTMT StatementHandle,
       if(isFirst)
       {
         isFirst= FALSE;
-        p+= my_snprintf(p, 5120 - strlen(StmtStr),
+        p+= _snprintf(p, 5120 - strlen(StmtStr),
           "SELECT \"%s\" AS TYPE_NAME, %d AS DATA_TYPE, %lu AS COLUMN_SIZE, \"%s\" AS LITERAL_PREFIX, "
           "\"%s\" AS LITERAL_SUFFIX, %s AS CREATE_PARAMS, %d AS NULLABLE, %d AS CASE_SENSITIVE, "
           "%d AS SEARCHABLE, %d AS UNSIGNED_ATTRIBUTE, %d AS FIXED_PREC_SCALE, %d AS AUTO_UNIQUE_VALUE, "
@@ -173,7 +173,7 @@ SQLRETURN MADB_GetTypeInfo(SQLHSTMT StatementHandle,
          TypeInfo[i].SqlDateTimeSub,TypeInfo[i].NumPrecRadix);
       }
       else
-          p+= my_snprintf(p, 5120 - strlen(StmtStr),
+          p+= _snprintf(p, 5120 - strlen(StmtStr),
           "UNION SELECT \"%s\", %d, %lu , \"%s\", "
           "\"%s\", %s, %d, %d, "
           "%d, %d, %d, %d, "
@@ -188,9 +188,7 @@ SQLRETURN MADB_GetTypeInfo(SQLHSTMT StatementHandle,
          TypeInfo[i].SqlDateTimeSub,TypeInfo[i].NumPrecRadix);
     }
   }
-  ret= Stmt->Methods->Prepare(Stmt, StmtStr, SQL_NTS);
-  if (SQL_SUCCEEDED(ret))
-    ret= Stmt->Methods->Execute(Stmt);
+  ret= Stmt->Methods->ExecDirect(Stmt, StmtStr, SQL_NTS);
   if (SQL_SUCCEEDED(ret))
   {
     MADB_FixColumnDataTypes(Stmt, gtiDefType);
