@@ -29,16 +29,21 @@
 
 #include <stdlib.h>
 
-#include <my_global.h>
-#include <my_sys.h>
 #include <mysql.h>
 
+#include <ma_legacy_helpers.h>
+
 #include <sql.h>
 #include <sqlext.h>
 #include <odbcinst.h>
 
 #include <errmsg.h>
 #include <string.h>
+#include <stdio.h>
+#include <math.h>
+#include <stddef.h>
+#include <assert.h>
+#include <time.h>
 
 #include <ma_odbc_version.h>
 
@@ -97,20 +102,19 @@ typedef struct
   SQLULEN      *BindOffsetPtr;
   SQLULEN       BindType;
   SQLSMALLINT   Count;
+  /* TODO: In IPD this is SQLUINTEGER* field */
   SQLULEN      *RowsProcessedPtr;
   /* Header end */
 } MADB_Header;
 
 typedef struct
 {
-	SQLLEN RowsetSize; /* for ODBC3 fetch operation */
 	SQLUINTEGER	BindSize;	/* size of each structure if using * Row-wise Binding */
 	SQLUSMALLINT	*RowOperationPtr;
 	SQLULEN		*RowOffsetPtr;
   MADB_ColBind *ColumnBind;
 	MYSQL_BIND *Bind;
 	SQLSMALLINT	Allocated;
-	SQLLEN		size_of_rowset_odbc2; /* for SQLExtendedFetch */
 } MADB_Ard;
 
 typedef struct
@@ -200,12 +204,12 @@ typedef struct
   MADB_Header Header;
   SQLINTEGER DescType;  /* SQL_ATTR_APP_ROW_DESC or SQL_ATTR_APP_PARAM_DESC */
   my_bool AppType;      /* Allocated by Application ? */
-  DYNAMIC_ARRAY Records;
-  DYNAMIC_ARRAY Stmts;
+  MADB_DynArray Records;
+  MADB_DynArray Stmts;
   MADB_Error Error;
   MADB_Dbc * Dbc;       /* Disconnect must automatically free allocated descriptors. Thus
                            descriptor has to know the connection it is allocated on */
-  LIST ListItem;        /* To store in the dbc */
+  MADB_List ListItem;        /* To store in the dbc */
   union {
     MADB_Ard Ard;
     MADB_Apd Apd;
@@ -222,29 +226,6 @@ struct st_ma_desc_fldid
 
 struct st_ma_stmt_methods;
 
-struct st_bind_column
-{
-  SQLUINTEGER TargetType;
-  SQLPOINTER TargetValuePtr;
-  SQLLEN BufferLength;
-  SQLLEN Utf8BufferLength;
-  SQLLEN *StrLen_or_Ind;
-  void *InternalBuffer;
-};
-
-struct st_bind_param
-{
-  SQLSMALLINT InputOutputType;
-  SQLSMALLINT ValueType;
-  SQLSMALLINT ParameterType;
-  SQLULEN ColumnSize;
-  SQLSMALLINT DecimalDigits; 
-  SQLPOINTER ParameterValuePtr;
-  SQLLEN BufferLength;
-  SQLLEN *StrLen_or_IndPtr;
-  void *InternalBuffer;
-};
-
 typedef struct 
 {
  	SQLLEN MaxRows;
@@ -266,6 +247,7 @@ typedef struct
 {
   char  *Name;
   SQLLEN Position;
+  SQLLEN RowsetSize;
 } MADB_Cursor;
 
 enum MADB_DaeType {MADB_DAE_NORMAL=0, MADB_DAE_ADD=1, MADB_DAE_UPDATE=2, MADB_DAE_DELETE=3};
@@ -276,11 +258,17 @@ enum MADB_DaeType {MADB_DAE_NORMAL=0, MADB_DAE_ADD=1, MADB_DAE_UPDATE=2, MADB_DA
 #define PARAM_IS_DAE(Len_Ptr) ((Len_Ptr) && (*(Len_Ptr) == SQL_DATA_AT_EXEC || *(Len_Ptr) <= SQL_LEN_DATA_AT_EXEC_OFFSET))
 #define DAE_DONE(Stmt_Hndl) ((Stmt_Hndl)->PutParam >= (Stmt_Hndl)->ParamCount)
 
+enum MADB_StmtState {MADB_SS_INITED= 0, MADB_SS_EMULATED= 1, MADB_SS_PREPARED= 2, MADB_SS_EXECUTED= 3, MADB_SS_OUTPARAMSFETCHED= 4};
 
-enum MADB_StmtState { MADB_SS_INITED= 0, MADB_SS_EMULATED= 1, MADB_SS_PREPARED= 2, MADB_SS_EXECUTED= 3 };
+#define STMT_WAS_PREPARED(Stmt_Hndl) (Stmt_Hndl->State >= MADB_SS_EMULATED)
+#define RESET_STMT_STATE(Stmt_Hndl) Stmt_Hndl->State= STMT_WAS_PREPARED(Stmt_Hndl) ?\
+  (Stmt_Hndl->State == MADB_SS_EMULATED ? MADB_SS_EMULATED : MADB_SS_PREPARED) :\
+  MADB_SS_INITED
 
-#define STMT_WAS_PREPARED(Stmt_Hndl) (Stmt_Hndl->State > MADB_SS_EMULATED)
-#define RESET_STMT_STATE(Stmt_Hndl) Stmt_Hndl->State= STMT_WAS_PREPARED(Stmt_Hndl) ? MADB_SS_PREPARED : MADB_SS_INITED
+
+typedef struct {
+  MADB_DynArray tokens;
+} MADB_QUERY;
 
 /* Struct used to define column type when driver has to fix it (in catalog functions + SQLGetTypeInfo) */
 typedef struct
@@ -292,10 +280,15 @@ typedef struct
 
 } MADB_ShortTypeInfo;
 
-/* Stmt struct needs definitions from my_parse.h */
-#include <ma_parse.h>
+typedef struct
+{
+  unsigned int  ArraySize;
+  my_bool       HasRowsToSkip;
 
-#define STMT_STRING(STMT) (STMT)->Query.Original
+} MADB_BulkOperationInfo;
+
+/* Stmt struct needs enum definition from my_parse.h, and it in its turn needs MADB_QUERY*/
+#include <ma_parse.h>
 
 struct st_ma_odbc_stmt
 {
@@ -306,9 +299,10 @@ struct st_ma_odbc_stmt
   MADB_Cursor               Cursor;
   MYSQL_STMT                *stmt;
   MYSQL_RES                 *metadata;
-  LIST                      ListItem;
-  MADB_QUERY                Query;
+  MADB_List                 ListItem;
+  MADB_QUERY                *Tokens;
   SQLSMALLINT               ParamCount;
+  my_bool                   isMultiQuery;
   enum MADB_DaeType         DataExecutionType;
   MYSQL_RES                 *DefaultsResult;
   int                       ArrayOffset;
@@ -316,28 +310,28 @@ struct st_ma_odbc_stmt
   int                       Status;
   MADB_DescRecord           *PutDataRec;
   MADB_Stmt                 *DaeStmt;
+  char                      *StmtString;
   MADB_Stmt                 *PositionedCursor;
   my_bool                   PositionedCommand;
   enum MADB_StmtState       State;
+  unsigned int              MultiStmtCount;
   MYSQL_STMT                **MultiStmts;
   unsigned int              MultiStmtNr;
   unsigned int              MultiStmtMaxParam;
   SQLLEN                    LastRowFetched;
-  struct st_bind_column     *bind_columns; /* ARD */
-  struct st_bind_param      *bind_params;
   MYSQL_BIND                *result;
   MYSQL_BIND                *params;
   int                       PutParam;
   my_bool                   RebindParams;
   my_bool                   bind_done;
-  SQLBIGINT                 AffectedRows;
+  long long                 AffectedRows;
   unsigned long             *CharOffset;
   unsigned long             *Lengths;
   char                      *TableName;
   char                      *CatalogName;
-  char                      TmpBuf[1]; /* for null bindings */
-  MYSQL_FIELD               *BulkFields;
   MADB_ShortTypeInfo        *ColsTypeFixArr;
+  MADB_BulkOperationInfo    Bulk;
+  enum enum_madb_query_type QueryType;
   /* Application Descriptors */
   MADB_Desc *Apd;
   MADB_Desc *Ard;
@@ -353,7 +347,7 @@ struct st_ma_odbc_stmt
 typedef struct st_ma_odbc_environment {
   MADB_Error Error;
   CRITICAL_SECTION cs;
-  LIST *Dbcs;
+  MADB_List *Dbcs;
   SQLUINTEGER Trace;
   SQLWCHAR *TraceFile;
   SQLINTEGER OdbcVersion;
@@ -367,7 +361,7 @@ typedef struct st_ma_odbc_environment {
 typedef struct st_client_charset
 {
   unsigned int CodePage;
-  CHARSET_INFO *cs_info;
+  MARIADB_CHARSET_INFO *cs_info;
 } Client_Charset;
 
 struct st_ma_odbc_connection
@@ -378,15 +372,11 @@ struct st_ma_odbc_connection
   MADB_Dsn *Dsn;
   struct st_ma_connection_methods *Methods;
   MADB_Error Error;
-
-  Client_Charset Charset;
-  Client_Charset *ConnOrSrcCharset; /* "Source" here stands for which charset Windows DM was using as source, when converted to unicode.
-                                  We have to use same charset to recode from unicode to get same string as application sent it.
-                                  For Unicode application that is the same as "Charset", or in case of ANSI on Windows - defaulst system codepage */
+  Client_Charset charset;
   char *DataBase;
-  LIST ListItem;
-  LIST *Stmts;
-  LIST *Descrs;
+  MADB_List ListItem;
+  MADB_List *Stmts;
+  MADB_List *Descrs;
   /* Attributes */
   SQLINTEGER AccessMode;
   my_bool IsAnsi;
@@ -422,14 +412,15 @@ typedef struct
 SQLRETURN DSNPrompt_Lookup(MADB_Prompt *prompt, const char *SetupLibName, MADB_Dbc *Dbc);
 int       DSNPrompt_Free  (MADB_Prompt *prompt);
 
+Client_Charset* GetDefaultOsCharset(Client_Charset *cc);
 int             InitClientCharset  (Client_Charset *cc, const char * name);
-void            CopyClientCharset(Client_Charset * Src, Client_Charset * Dst);
 void            CloseClientCharset(Client_Charset *cc);
 
 /* Default precision of SQL_NUMERIC */
 #define MADB_DEFAULT_PRECISION 38
 #define BINARY_CHARSETNR       63
-
+/* Inexistent param id */
+#define MADB_NOPARAM           -1
 /* Macros to guard communications with the server.
    TODO: make it(locking) optional depending on designated connection string option */
 #define LOCK_MARIADB(Dbc)   EnterCriticalSection(&(Dbc)->cs)
@@ -441,10 +432,9 @@ void            CloseClientCharset(Client_Charset *cc);
 #define RETURN_ERROR_OR_CONTINUE(sqlreturn_func_call) {\
   SQLRETURN rc= (sqlreturn_func_call);\
   if (!SQL_SUCCEEDED(rc)) return rc;\
-}
+} while(0)
 
 #include <ma_error.h>
-#include <ma_parse.h>
 #include <ma_info.h>
 #include <ma_environment.h>
 #include <ma_connection.h>
@@ -456,6 +446,8 @@ void            CloseClientCharset(Client_Charset *cc);
 #include <ma_driver.h>
 #include <ma_helper.h>
 #include <ma_server.h>
+#include <ma_typeconv.h>
+#include <ma_bulk.h>
 
 /* SQLFunction calls inside MariaDB Connector/ODBC needs to be mapped,
  * on non Windows platforms these function calls will call the driver
@@ -475,12 +467,6 @@ SQLRETURN MA_SQLBindParameter(SQLHSTMT StatementHandle,
     SQLLEN BufferLength,
     SQLLEN *StrLen_or_IndPtr);
 
-SQLRETURN MA_SQLExecDirect(MADB_Stmt *Stmt,
-    SQLCHAR *StatementText,
-    SQLINTEGER TextLength);
-
-SQLRETURN MA_SQLExecute(MADB_Stmt *Stmt);
-
 SQLRETURN MA_SQLFreeStmt(SQLHSTMT StatementHandle,
     SQLUSMALLINT Option);
 
@@ -496,10 +482,6 @@ SQLRETURN MA_SQLPrepare(MADB_Stmt *Stmt,
 
 SQLRETURN MA_SQLCancel(SQLHSTMT StatementHandle);
 
-SQLRETURN MA_SQLFetchScroll(SQLHSTMT StatementHandle,
-    SQLSMALLINT FetchOrientation,
-    SQLLEN FetchOffset);
-
 SQLRETURN MA_SQLGetDiagRec(SQLSMALLINT HandleType,
     SQLHANDLE Handle,
     SQLSMALLINT RecNumber,
