@@ -1,5 +1,5 @@
 /************************************************************************************
-   Copyright (C) 2014,2015 MariaDB Corporation AB
+   Copyright (C) 2014,2016 MariaDB Corporation AB
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -29,6 +29,21 @@ extern Client_Charset utf8;
 char LogFile[256];
 
 
+char *strndup(const char *s, size_t n)
+{
+  size_t len= MIN(strlen(s), n);
+  char  *res= (char*)malloc(len + 1);
+
+  if (res != NULL)
+  {
+    memcpy(res, s, len);
+    res[len]= '\0';
+  }
+
+  return res;
+}
+
+
 const char* GetDefaultLogDir()
 {
   const char *DefaultLogDir= "c:";
@@ -79,7 +94,7 @@ int DSNPrompt_Free(MADB_Prompt *prompt)
 }
 
 
-SQLWCHAR *MADB_ConvertToWchar(char *Ptr, SQLLEN PtrLength, Client_Charset* cc)
+SQLWCHAR *MADB_ConvertToWchar(const char *Ptr, SQLLEN PtrLength, Client_Charset* cc)
 {
   SQLWCHAR *WStr= NULL;
   int Length;
@@ -108,10 +123,10 @@ SQLWCHAR *MADB_ConvertToWchar(char *Ptr, SQLLEN PtrLength, Client_Charset* cc)
   return WStr;
 }
 
-/* {{{ MADB_ConvertFromWChar
-       Length gets number of written bytes including TN (if WstrCharLen == -1 or SQL_NTS or if WstrCharLen includes
-       TN in the Wstr) */
-char *MADB_ConvertFromWChar(SQLWCHAR *Wstr, SQLINTEGER WstrCharLen, SQLULEN *Length/*Bytes*/, Client_Charset *cc, BOOL *Error)
+  /* {{{ MADB_ConvertFromWChar
+  Length gets number of written bytes including TN (if WstrCharLen == -1 or SQL_NTS or if WstrCharLen includes
+  TN in the Wstr) */
+char *MADB_ConvertFromWChar(const SQLWCHAR *Wstr, SQLINTEGER WstrCharLen, SQLULEN *Length/*Bytes*/, Client_Charset *cc, BOOL *Error)
 {
   char *AscStr;
   int AscLen, AllocLen;
@@ -145,7 +160,7 @@ char *MADB_ConvertFromWChar(SQLWCHAR *Wstr, SQLINTEGER WstrCharLen, SQLULEN *Len
 /* }}} */
 
 /* Required Length without or with TN(if IsNull is TRUE, or AnsiLength == -1 or SQL_NTS) is put to LenghtIndicator*/
-int MADB_ConvertAnsi2Unicode(Client_Charset *cc, char *AnsiString, SQLLEN AnsiLength, 
+int MADB_ConvertAnsi2Unicode(Client_Charset *cc, const char *AnsiString, SQLLEN AnsiLength,
                              SQLWCHAR *UnicodeString, SQLLEN UnicodeLength, 
                              SQLLEN *LengthIndicator, BOOL IsNull, MADB_Error *Error)
 {
@@ -209,7 +224,7 @@ end:
    then simply SrcLength is returned. 
    If Dest is not NULL, and DestLenth is 0, then error*/
 SQLLEN MADB_SetString(Client_Charset* cc, void *Dest, SQLULEN DestLength,
-                      char *Src, SQLLEN SrcLength/*bytes*/, MADB_Error *Error)
+                      const char *Src, SQLLEN SrcLength/*bytes*/, MADB_Error *Error)
 {
   char  *p= (char *)Dest;
   SQLLEN Length= 0;
