@@ -1,5 +1,5 @@
 /************************************************************************************
-   Copyright (C) 2013,2018 MariaDB Corporation AB
+   Copyright (C) 2013 SkySQL AB
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -18,22 +18,6 @@
 *************************************************************************************/
  #include <ma_odbc.h>
 
-
-/* {{{ MADB_StmtResetResultStructures */
-void MADB_StmtResetResultStructures(MADB_Stmt *Stmt)
-{
-  Stmt->CharOffset= (unsigned long *)MADB_REALLOC((char *)Stmt->CharOffset,
-    sizeof(long) * mysql_stmt_field_count(Stmt->stmt));
-  memset(Stmt->CharOffset, 0, sizeof(long) * mysql_stmt_field_count(Stmt->stmt));
-  Stmt->Lengths= (unsigned long *)MADB_REALLOC((char *)Stmt->Lengths,
-    sizeof(long) * mysql_stmt_field_count(Stmt->stmt));
-  memset(Stmt->Lengths, 0, sizeof(long) * mysql_stmt_field_count(Stmt->stmt));
-
-  Stmt->LastRowFetched= 0;
-  MADB_STMT_RESET_CURSOR(Stmt);
-}
-/* }}} */
-
 /* {{{ MADB_StmtDataSeek */
 SQLRETURN MADB_StmtDataSeek(MADB_Stmt *Stmt, my_ulonglong FetchOffset)
 {
@@ -43,6 +27,7 @@ SQLRETURN MADB_StmtDataSeek(MADB_Stmt *Stmt, my_ulonglong FetchOffset)
   {
    return SQL_NO_DATA_FOUND;
   }
+
   mysql_stmt_data_seek(Stmt->stmt, FetchOffset);
 
   return SQL_SUCCESS;  
@@ -55,21 +40,17 @@ SQLRETURN MADB_StmtMoreResults(MADB_Stmt *Stmt)
   SQLRETURN ret= SQL_SUCCESS;
   if (!Stmt->stmt)
   {
-    MADB_SetError(&Stmt->Error, MADB_ERR_08S01, NULL, 0);
-    return Stmt->Error.ReturnValue;
+    return MADB_SetError(&Stmt->Error, MADB_ERR_08S01, NULL, 0);
   }
 
   if (Stmt->MultiStmts)
   {
-    if (Stmt->MultiStmtNr == STMT_COUNT(Stmt->Query) - 1)
-    {
+    if (Stmt->MultiStmtNr == Stmt->MultiStmtCount - 1)
       return SQL_NO_DATA;
-    }
-
-    ++Stmt->MultiStmtNr;
-
-    MADB_InstallStmt(Stmt, Stmt->MultiStmts[Stmt->MultiStmtNr]);
 
+    Stmt->MultiStmtNr++;
+    Stmt->stmt= Stmt->MultiStmts[Stmt->MultiStmtNr];
+    Stmt->AffectedRows= mysql_stmt_affected_rows(Stmt->stmt);
     return SQL_SUCCESS;
   }
 
@@ -88,50 +69,58 @@ SQLRETURN MADB_StmtMoreResults(MADB_Stmt *Stmt)
   }
 
   if (mysql_stmt_more_results(Stmt->stmt))
-  {
     mysql_stmt_free_result(Stmt->stmt);
-  }
   else
-  {
     return SQL_NO_DATA;
-  }
   
   LOCK_MARIADB(Stmt->Connection);
-  if (mysql_stmt_next_result(Stmt->stmt) > 0)
+  if (mysql_stmt_next_result(Stmt->stmt) ||
+      !Stmt->stmt->field_count)
   {
     UNLOCK_MARIADB(Stmt->Connection);
-    return MADB_SetNativeError(&Stmt->Error, SQL_HANDLE_STMT, Stmt->stmt);
+    return SQL_NO_DATA;
   }
-  
-  /* We can't have it in MADB_StmtResetResultStructures, as it breaks dyn_cursor functionality.
-     Thus we free-ing bind structs on move to new result only */
-  MADB_FREE(Stmt->result);
-  MADB_StmtResetResultStructures(Stmt);
 
-  if (mysql_stmt_field_count(Stmt->stmt) == 0)
+  MADB_STMT_RESET_CURSOR(Stmt);
+
+  MADB_DescSetIrdMetadata(Stmt, mysql_fetch_fields(FetchMetadata(Stmt)), mysql_stmt_field_count(Stmt->stmt));
+
+  if (Stmt->Connection->mariadb->server_status & SERVER_PS_OUT_PARAMS)
   {
-    Stmt->AffectedRows= mysql_stmt_affected_rows(Stmt->stmt);
+    Stmt->State= MADB_SS_OUTPARAMSFETCHED;
+    ret= Stmt->Methods->GetOutParams(Stmt, 0);
   }
   else
   {
-    MADB_DescSetIrdMetadata(Stmt, mysql_fetch_fields(FetchMetadata(Stmt)), mysql_stmt_field_count(Stmt->stmt));
+    if (Stmt->Options.CursorType != SQL_CURSOR_FORWARD_ONLY)
+    {
+      mysql_stmt_store_result(Stmt->stmt);
+    }
+  }
+  UNLOCK_MARIADB(Stmt->Connection);
 
-    if (Stmt->Connection->mariadb->server_status & SERVER_PS_OUT_PARAMS)
-    {
-      Stmt->State= MADB_SS_OUTPARAMSFETCHED;
-      ret= Stmt->Methods->GetOutParams(Stmt, 0);
-    }
-    else
-    {
-      if (Stmt->Options.CursorType != SQL_CURSOR_FORWARD_ONLY)
-      {
-        mysql_stmt_store_result(Stmt->stmt);
-        mysql_stmt_data_seek(Stmt->stmt, 0);
-      }
-    }
-    UNLOCK_MARIADB(Stmt->Connection);
+  if (/* mysql_stmt_field_count(Stmt->stmt) && */
+        Stmt->Options.CursorType != SQL_CURSOR_FORWARD_ONLY)
+  {
+    mysql_stmt_data_seek(Stmt->stmt, 0);
   }
 
   return ret;
 }
 /* }}} */
+
+/* {{{ MADB_RecordsToFetch */
+SQLULEN MADB_RowsToFetch(MADB_Cursor *Cursor, SQLULEN ArraySize, unsigned long long RowsInResultst)
+{
+  SQLLEN  Position= Cursor->Position >= 0 ? Cursor->Position : 0;
+  SQLULEN result= ArraySize;
+
+  Cursor->RowsetSize= ArraySize;
+
+  if (Position + ArraySize > RowsInResultst)
+    result= (SQLULEN)(RowsInResultst - Position);
+
+  return result >= 0 ? result : 1;
+}
+/* }}} */
+
