@@ -26,8 +26,8 @@
 
 struct st_ma_stmt_methods
 {
-  SQLRETURN (*Prepare)(MADB_Stmt *Stmt, char *StatementText, SQLINTEGER TextLength);
-  SQLRETURN (*Execute)(MADB_Stmt *Stmt);
+  SQLRETURN (*Prepare)(MADB_Stmt *Stmt, char *StatementText, SQLINTEGER TextLength, BOOL ExecDirect);
+  SQLRETURN (*Execute)(MADB_Stmt *Stmt, BOOL ExecDirect);
   SQLRETURN (*Fetch)(MADB_Stmt *Stmt);
   SQLRETURN (*BindColumn)(MADB_Stmt *Stmt, SQLUSMALLINT ColumnNumber, SQLSMALLINT TargetType,
                      SQLPOINTER TargetValuePtr, SQLLEN BufferLength, SQLLEN *StrLen_or_Ind);
@@ -105,6 +105,7 @@ struct st_ma_stmt_methods
 SQLRETURN    MADB_StmtInit          (MADB_Dbc *Connection, SQLHANDLE *pHStmt);
 SQLUSMALLINT MapColAttributeDescType(SQLUSMALLINT FieldIdentifier);
 MYSQL_RES*   FetchMetadata          (MADB_Stmt *Stmt);
+SQLRETURN    MADB_DoExecute         (MADB_Stmt *Stmt, BOOL ExecDirect);
 
 #define MADB_MAX_CURSOR_NAME 64 * 3 + 1
 #define MADB_CHECK_STMT_HANDLE(a,b)\
@@ -114,6 +115,8 @@ MYSQL_RES*   FetchMetadata          (MADB_Stmt *Stmt);
 #define MADB_STMT_COLUMN_COUNT(aStmt) (aStmt)->Ird->Header.Count
 #define MADB_STMT_PARAM_COUNT(aStmt)  (aStmt)->ParamCount
 #define MADB_POSITIONED_COMMAND(aStmt) ((aStmt)->PositionedCommand && (aStmt)->PositionedCursor)
+/* So far we always use all fields for index. Once that is changed, this should be changed as well */
+#define MADB_POS_COMM_IDX_FIELD_COUNT(aStmt) MADB_STMT_COLUMN_COUNT((aStmt)->PositionedCursor)
 #define MADB_STMT_RESET_CURSOR(aStmt) (aStmt)->Cursor.Position= -1; 
 
 #define MADB_TRANSFER_OCTET_LENGTH\
@@ -187,8 +190,8 @@ MYSQL_RES*   FetchMetadata          (MADB_Stmt *Stmt);
 
 /************** SQLColumns       *************/
 #define MADB_COLUMN_SIZE\
-    "CAST(CASE" \
-    "  WHEN DATA_TYPE = 'bit' THEN @ColSize:=((NUMERIC_PRECISION + 7) / 8) "\
+    "CASE" \
+    "  WHEN DATA_TYPE = 'bit' THEN @ColSize:=CAST(((NUMERIC_PRECISION + 7) / 8) AS SIGNED) "\
     "  WHEN DATA_TYPE in ('tinyint', 'smallint', 'year', 'mediumint', 'int',"\
                          "'bigint', 'decimal', 'double') THEN @ColSize:=NUMERIC_PRECISION "\
     "  WHEN DATA_TYPE = 'float' THEN @ColSize:=7"\
@@ -196,7 +199,7 @@ MYSQL_RES*   FetchMetadata          (MADB_Stmt *Stmt);
     "  WHEN DATA_TYPE='time' THEN @ColSize:=8"\
     "  WHEN DATA_TYPE in ('timestamp', 'datetime') THEN @ColSize:=19 "\
     "  ELSE @ColSize:=CHARACTER_MAXIMUM_LENGTH "\
-  "END AS SIGNED)"
+  "END "
 
 #define MADB_CATALOG_COLUMNSp1 "SELECT TABLE_SCHEMA AS TABLE_CAT, NULL AS TABLE_SCHEM, TABLE_NAME, COLUMN_NAME, "
 #define MADB_CATALOG_COLUMNSp3  ", UCASE(IF(COLUMN_TYPE LIKE '%%(%%)%%',  CONCAT(SUBSTRING(COLUMN_TYPE,1, LOCATE('(',COLUMN_TYPE) - 1 ), SUBSTRING(COLUMN_TYPE,1+locate(')',COLUMN_TYPE))), COLUMN_TYPE )) AS TYPE_NAME, "\
