@@ -1,5 +1,5 @@
 /************************************************************************************
-   Copyright (C) 2013,2016 MariaDB Corporation AB
+   Copyright (C) 2013,2017 MariaDB Corporation AB
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -18,7 +18,7 @@
 *************************************************************************************/
 #include <ma_odbc.h>
 
-extern CHARSET_INFO*  utf16;
+extern MARIADB_CHARSET_INFO*  utf16;
 
 char *MADB_GetTableName(MADB_Stmt *Stmt)
 {
@@ -74,21 +74,21 @@ char *MADB_GetCatalogName(MADB_Stmt *Stmt)
   return CatalogName;
 }
 
-my_bool MADB_DynStrAppendQuoted(DYNAMIC_STRING *DynString, char *String)
+my_bool MADB_DynStrAppendQuoted(MADB_DynString *DynString, char *String)
 {
-  if (dynstr_append(DynString, "`") ||
-      dynstr_append(DynString, String) ||
-      dynstr_append(DynString, "`"))
+  if (MADB_DynstrAppend(DynString, "`") ||
+      MADB_DynstrAppend(DynString, String) ||
+      MADB_DynstrAppend(DynString, "`"))
     return TRUE;
   return FALSE;
 }
 
-my_bool MADB_DynStrUpdateSet(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
+my_bool MADB_DynStrUpdateSet(MADB_Stmt *Stmt, MADB_DynString *DynString)
 {
   int             i, IgnoredColumns= 0;
   MADB_DescRecord *Record;
 
-  if (dynstr_append(DynString, " SET "))
+  if (MADB_DynstrAppend(DynString, " SET "))
   {
     MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
     return TRUE;
@@ -107,7 +107,7 @@ my_bool MADB_DynStrUpdateSet(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
       continue;
     }
     
-    if ((i - IgnoredColumns) && dynstr_append(DynString, ","))
+    if ((i - IgnoredColumns) && MADB_DynstrAppend(DynString, ","))
     {
       MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
       return TRUE;
@@ -117,7 +117,7 @@ my_bool MADB_DynStrUpdateSet(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
       MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
       return TRUE;
     }
-    if (dynstr_append(DynString, "=?"))
+    if (MADB_DynstrAppend(DynString, "=?"))
     {
       MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
       return TRUE;
@@ -131,14 +131,14 @@ my_bool MADB_DynStrUpdateSet(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
   return FALSE;
 }
 
-my_bool MADB_DynStrInsertSet(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
+my_bool MADB_DynStrInsertSet(MADB_Stmt *Stmt, MADB_DynString *DynString)
 {
-  DYNAMIC_STRING  ColVals;
+  MADB_DynString  ColVals;
   int             i, NeedComma= 0;
   MADB_DescRecord *Record;
 
-  init_dynamic_string(&ColVals, "VALUES (", 32, 32);
-  if (dynstr_append(DynString, " (") )
+  MADB_InitDynamicString(&ColVals, "VALUES (", 32, 32);
+  if (MADB_DynstrAppend(DynString, " ("))
   {
     goto dynerror;
     
@@ -149,45 +149,44 @@ my_bool MADB_DynStrInsertSet(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
   for (i= 0; i < MADB_STMT_COLUMN_COUNT(Stmt); i++)
   {
     Record= MADB_DescGetInternalRecord(Stmt->Ard, i, MADB_DESC_READ);
-    
     if (!Record->inUse || MADB_ColumnIgnoredInAllRows(Stmt->Ard, Record) == TRUE)
     {
       continue;
     }
 
     if ((NeedComma) && 
-        (dynstr_append(DynString, ",") || dynstr_append(&ColVals, ",")))
+        (MADB_DynstrAppend(DynString, ",") || MADB_DynstrAppend(&ColVals, ",")))
       goto dynerror;
 
     if (MADB_DynStrAppendQuoted(DynString, Stmt->stmt->fields[i].org_name) ||
-        dynstr_append(&ColVals, "?"))
+        MADB_DynstrAppend(&ColVals, "?"))
        goto dynerror;
 
     NeedComma= 1;
   }
-  if (dynstr_append(DynString, ") " ) ||
-      dynstr_append(&ColVals, ")") ||
-      dynstr_append(DynString, ColVals.str))
+  if (MADB_DynstrAppend(DynString, ") " ) ||
+      MADB_DynstrAppend(&ColVals, ")") ||
+      MADB_DynstrAppend(DynString, ColVals.str))
     goto dynerror;
-  dynstr_free(&ColVals);
+  MADB_DynstrFree(&ColVals);
   return FALSE;
 dynerror:
   MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
-  dynstr_free(&ColVals);
+  MADB_DynstrFree(&ColVals);
   return TRUE;
 }
 
-my_bool MADB_DynStrGetColumns(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
+my_bool MADB_DynStrGetColumns(MADB_Stmt *Stmt, MADB_DynString *DynString)
 {
   unsigned int i;
-  if (dynstr_append(DynString, " ("))
+  if (MADB_DynstrAppend(DynString, " ("))
   {
     MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
     return TRUE;
   }
   for (i=0; i < mysql_stmt_field_count(Stmt->stmt); i++)
   {
-    if (i && dynstr_append(DynString, ", "))
+    if (i && MADB_DynstrAppend(DynString, ", "))
     {
       MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
       return TRUE;
@@ -198,7 +197,7 @@ my_bool MADB_DynStrGetColumns(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
       return TRUE;
     }
   }
-  if (dynstr_append(DynString, " )"))
+  if (MADB_DynstrAppend(DynString, " )"))
   {
     MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
     return TRUE;
@@ -206,19 +205,19 @@ my_bool MADB_DynStrGetColumns(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
   return FALSE;
 }
 
-my_bool MADB_DynStrGetWhere(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString, char *TableName, my_bool ParameterMarkers)
+my_bool MADB_DynStrGetWhere(MADB_Stmt *Stmt, MADB_DynString *DynString, char *TableName, my_bool ParameterMarkers)
 {
   int UniqueCount=0, PrimaryCount= 0;
-  unsigned int i;
-  int Flag= 0;
+  int i, Flag= 0;
   char *Column= NULL;
   SQLLEN StrLength;
 
-  for (i=0; i < mysql_stmt_field_count(Stmt->stmt);i++)
+  for (i= 0; i < MADB_STMT_COLUMN_COUNT(Stmt); i++)
   {
-    if (Stmt->stmt->fields[i].flags & PRI_KEY_FLAG)
+    MYSQL_FIELD *field= mysql_fetch_field_direct(FetchMetadata(Stmt), i);
+    if (field->flags & PRI_KEY_FLAG)
       PrimaryCount++;
-    if (Stmt->stmt->fields[i].flags & UNIQUE_KEY_FLAG)
+    if (field->flags & UNIQUE_KEY_FLAG)
       UniqueCount++;
   }
   /* We need to use all columns, otherwise it will be difficult to map fields for Positioned Update */
@@ -236,29 +235,30 @@ my_bool MADB_DynStrGetWhere(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString, char *Ta
     int       FieldCount= 0;
 
     MA_SQLAllocHandle(SQL_HANDLE_STMT, Stmt->Connection, (SQLHANDLE*)&CountStmt);
-    my_snprintf(StmtStr, 256, "SELECT * FROM `%s` LIMIT 0", TableName);
+    _snprintf(StmtStr, 256, "SELECT * FROM `%s` LIMIT 0", TableName);
     CountStmt->Methods->ExecDirect(CountStmt, (SQLCHAR *)StmtStr, SQL_NTS);
     FieldCount= mysql_stmt_field_count(((MADB_Stmt *)CountStmt)->stmt);
     CountStmt->Methods->StmtFree(CountStmt, SQL_DROP);
 
-    if (FieldCount != mysql_stmt_field_count(Stmt->stmt))
+    if (FieldCount != MADB_STMT_COLUMN_COUNT(Stmt))
     {
       MADB_SetError(&Stmt->Error, MADB_ERR_S1000, "Can't build index for update/delete", 0);
       return TRUE;
     }
   }
-  if (dynstr_append(DynString, " WHERE 1"))
+  if (MADB_DynstrAppend(DynString, " WHERE 1"))
     goto memerror;
-  for (i=0; i < mysql_stmt_field_count(Stmt->stmt);i++)
+  for (i= 0; i < MADB_STMT_COLUMN_COUNT(Stmt); i++)
   {
-    if (Stmt->stmt->fields[i].flags & Flag || !Flag)
+    MYSQL_FIELD *field= mysql_fetch_field_direct(Stmt->metadata, i);
+    if (field->flags & Flag || !Flag)
     {
-      if (dynstr_append(DynString, " AND ") ||
-          MADB_DynStrAppendQuoted(DynString, Stmt->stmt->fields[i].org_name))
+      if (MADB_DynstrAppend(DynString, " AND ") ||
+          MADB_DynStrAppendQuoted(DynString, field->org_name))
           goto memerror;
       if (ParameterMarkers)
       {
-        if (dynstr_append(DynString, "=?"))
+        if (MADB_DynstrAppend(DynString, "=?"))
           goto memerror;
       }
       else
@@ -270,50 +270,56 @@ my_bool MADB_DynStrGetWhere(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString, char *Ta
         }
         if (StrLength < 0)
         {
-           if (dynstr_append(DynString, " IS NULL"))
+           if (MADB_DynstrAppend(DynString, " IS NULL"))
              goto memerror;
         }
         else
         {
           Column= MADB_CALLOC(StrLength + 1);
           Stmt->Methods->GetData(Stmt,i+1, SQL_C_CHAR, Column, StrLength + 1, NULL, TRUE);
-          if (dynstr_append(DynString, "= '") ||
-                 dynstr_append(DynString, Column) ||
-                 dynstr_append(DynString, "'"))
-          goto memerror;
+          if (MADB_DynstrAppend(DynString, "= '") ||
+            MADB_DynstrAppend(DynString, Column) ||
+            MADB_DynstrAppend(DynString, "'"))
+          {
+            goto memerror;
+          }
           MADB_FREE(Column);
           Column= NULL;
         }
       }
     }
   }
-  if (dynstr_append(DynString, " LIMIT 1"))
+  if (MADB_DynstrAppend(DynString, " LIMIT 1"))
     goto memerror;
   MADB_FREE(Column);
+
   return FALSE;
+
 memerror:
   MADB_FREE(Column);
   MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
+
   return TRUE;
 }
 
-my_bool MADB_DynStrGetValues(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
+
+my_bool MADB_DynStrGetValues(MADB_Stmt *Stmt, MADB_DynString *DynString)
 {
   unsigned int i;
-  if (dynstr_append(DynString, " VALUES("))
+  if (MADB_DynstrAppend(DynString, " VALUES("))
   {
     MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
     return TRUE;
   }
   for (i=0; i < mysql_stmt_field_count(Stmt->stmt); i++)
   {
-    if (dynstr_append(DynString, (i) ? ",?" : "?"))
+    if (MADB_DynstrAppend(DynString, (i) ? ",?" : "?"))
     {
       MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
       return TRUE;
     }
   }
-  if (dynstr_append(DynString, ")"))
+  if (MADB_DynstrAppend(DynString, ")"))
   {
     MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
     return TRUE;
@@ -338,10 +344,10 @@ char *MADB_GetInsertStatement(MADB_Stmt *Stmt)
     goto error;
   p= StmtStr;
   
-  p+= my_snprintf(StmtStr, 1024, "INSERT INTO `%s` (", TableName);
+  p+= _snprintf(StmtStr, 1024, "INSERT INTO `%s` (", TableName);
   for (i=0; i < mysql_stmt_field_count(Stmt->stmt); i++)
   {
-    if (strlen(StmtStr) > Length - 100)
+    if (strlen(StmtStr) > Length - NAME_LEN - 4/* comma + 2 ticks + terminating NULL */)
     {
       Length+= 1024;
       if (!(StmtStr= MADB_REALLOC(StmtStr, Length)))
@@ -350,23 +356,26 @@ char *MADB_GetInsertStatement(MADB_Stmt *Stmt)
         goto error;
       }
     }
-    p+= my_snprintf(p, Length - strlen(StmtStr), "%s`%s`", (i==0) ? "" : ",", Stmt->stmt->fields[i].org_name);
+    p+= _snprintf(p, Length - strlen(StmtStr), "%s`%s`", (i==0) ? "" : ",", Stmt->stmt->fields[i].org_name);
   }
-  p+= my_snprintf(p, Length - strlen(StmtStr), ") VALUES (");
+  p+= _snprintf(p, Length - strlen(StmtStr), ") VALUES (");
+
+  if (strlen(StmtStr) > Length - mysql_stmt_field_count(Stmt->stmt)*2 - 1)/* , and ? for each column  + (- 1 comma for 1st column + closing ')')
+                                                                            + terminating NULL */
+  {
+    Length= strlen(StmtStr) + mysql_stmt_field_count(Stmt->stmt)*2 + 1;
+    if (!(StmtStr= MADB_REALLOC(StmtStr, Length)))
+    {
+      MADB_SetError(&Stmt->Error, MADB_ERR_HY013, NULL, 0);
+      goto error;
+    }
+  }
+
   for (i=0; i < mysql_stmt_field_count(Stmt->stmt); i++)
   {
-    if (strlen(StmtStr) > Length - 100)
-    {
-      Length+= 1024;
-      if (!(StmtStr= MADB_REALLOC(StmtStr, Length)))
-      {
-        MADB_SetError(&Stmt->Error, MADB_ERR_HY013, NULL, 0);
-        goto error;
-      }
-    }
-    p+= my_snprintf(p, Length - strlen(StmtStr), "%s?", (i==0) ? "" : ",");
+    p+= _snprintf(p, Length - strlen(StmtStr), "%s?", (i==0) ? "" : ",");
   }
-  p+= my_snprintf(p, Length - strlen(StmtStr), ")");
+  p+= _snprintf(p, Length - strlen(StmtStr), ")");
   return StmtStr;
 
 error:
@@ -440,7 +449,7 @@ int InitClientCharset(Client_Charset *cc, const char * name)
 {
   /* There is no legal charset names longer than 31 chars */
   char lowered[32];
-  cc->cs_info= mysql_get_charset_by_name(MADB_ToLower(name, lowered, sizeof(lowered)));
+  cc->cs_info= mariadb_get_charset_by_name(MADB_ToLower(name, lowered, sizeof(lowered)));
 
   if (cc->cs_info == NULL)
   {
@@ -455,7 +464,8 @@ int InitClientCharset(Client_Charset *cc, const char * name)
 
 Client_Charset* GetDefaultOsCharset(Client_Charset *cc)
 {
-  if (InitClientCharset(cc, madb_get_os_character_set()))
+  MARIADB_CHARSET_INFO *cs= mariadb_get_charset_by_name("auto");
+  if (InitClientCharset(cc, cs->csname))
   {
     return NULL;
   }
@@ -469,7 +479,7 @@ void CloseClientCharset(Client_Charset *cc)
 
 
 /* Hmmm... Length in characters is SQLLEN, octet length SQLINTEGER */
-SQLLEN MbstrOctetLen(char *str, SQLLEN *CharLen, CHARSET_INFO *cs)
+SQLLEN MbstrOctetLen(const char *str, SQLLEN *CharLen, MARIADB_CHARSET_INFO *cs)
 {
   SQLLEN result= 0, inChars= *CharLen;
 
@@ -509,10 +519,10 @@ SQLLEN MbstrOctetLen(char *str, SQLLEN *CharLen, CHARSET_INFO *cs)
 
 
 /* Number of characters in given number of bytes */
-SQLLEN MbstrCharLen(char *str, SQLINTEGER OctetLen, CHARSET_INFO *cs)
+SQLLEN MbstrCharLen(const char *str, SQLINTEGER OctetLen, MARIADB_CHARSET_INFO *cs)
 {
   SQLLEN       result= 0;
-  char        *ptr= str;
+  const char   *ptr= str;
   unsigned int charlen;
 
   if (str)
@@ -575,20 +585,40 @@ SQLINTEGER SqlwcsCharLen(SQLWCHAR *str, SQLLEN octets)
 }
 
 
-/* Length in SQLWCHAR units*/
-SQLINTEGER SqlwcsLen(SQLWCHAR *str)
+/* Length in SQLWCHAR units
+   @buff_length[in] - size of the str buffer or negative number  */
+SQLLEN SqlwcsLen(SQLWCHAR *str, SQLLEN buff_length)
 {
   SQLINTEGER result= 0;
 
   if (str)
   {
-    while (*str)
+    /* If buff_length is negative - we will never hit 1st condition, otherwise we hit it after last character
+       of the buffer is processed */
+    while ((--buff_length) != -1 && *str)
     {
       ++result;
-      /* str+= (utf16->mb_charlen(*str))/sizeof(SQLWCHAR)); */
       ++str;
     }
   }
   return result;
 }
 
+/* Length of a string with respect to specified buffer size
+@buff_length[in] - size of the str buffer or negative number  */
+SQLLEN SafeStrlen(SQLCHAR *str, SQLLEN buff_length)
+{
+  SQLINTEGER result= 0;
+
+  if (str)
+  {
+    /* If buff_length is negative - we will never hit 1st condition, otherwise we hit it after last character
+    of the buffer is processed */
+    while ((--buff_length) != -1 && *str)
+    {
+      ++result;
+      ++str;
+    }
+  }
+  return result;
+}
\ No newline at end of file
