@@ -306,7 +306,7 @@ SQLRETURN MA_SQLCancel(SQLHSTMT StatementHandle)
       goto end;
     }
     
-    my_snprintf(StmtStr, 30, "KILL QUERY %ld", mysql_thread_id(Kill));
+    _snprintf(StmtStr, 30, "KILL QUERY %ld", mysql_thread_id(Kill));
     if (mysql_query(MariaDb, StmtStr))
     {
       mysql_close(MariaDb);
@@ -539,10 +539,10 @@ SQLRETURN SQL_API SQLColumnPrivilegesW(SQLHSTMT StatementHandle,
 
   MDBUG_C_ENTER(Stmt->Connection, "SQLColumnPrivilegesW");
 
-  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpTable= MADB_ConvertFromWChar(TableName, NameLength3, &CpLength3, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpColumn= MADB_ConvertFromWChar(ColumnName, NameLength4, &CpLength4, Stmt->Connection->ConnOrSrcCharset, NULL);
+  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, &Stmt->Connection->charset, NULL);
+  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, &Stmt->Connection->charset, NULL);
+  CpTable= MADB_ConvertFromWChar(TableName, NameLength3, &CpLength3, &Stmt->Connection->charset, NULL);
+  CpColumn= MADB_ConvertFromWChar(ColumnName, NameLength4, &CpLength4, &Stmt->Connection->charset, NULL);
 
   ret= Stmt->Methods->ColumnPrivileges(Stmt, CpCatalog, (SQLSMALLINT)CpLength1, CpSchema, (SQLSMALLINT)CpLength2,
                                        CpTable, (SQLSMALLINT)CpLength3, CpColumn, (SQLSMALLINT)CpLength4);
@@ -609,10 +609,10 @@ SQLRETURN SQL_API SQLColumnsW(SQLHSTMT StatementHandle,
 
   MDBUG_C_ENTER(Stmt->Connection, "SQLColumns");
 
-  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpSchema=  MADB_ConvertFromWChar(SchemaName,  NameLength2, &CpLength2, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpTable=   MADB_ConvertFromWChar(TableName,   NameLength3, &CpLength3, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpColumn=  MADB_ConvertFromWChar(ColumnName,  NameLength4, &CpLength4, Stmt->Connection->ConnOrSrcCharset, NULL);
+  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, &Stmt->Connection->charset, NULL);
+  CpSchema=  MADB_ConvertFromWChar(SchemaName,  NameLength2, &CpLength2, &Stmt->Connection->charset, NULL);
+  CpTable=   MADB_ConvertFromWChar(TableName,   NameLength3, &CpLength3, &Stmt->Connection->charset, NULL);
+  CpColumn=  MADB_ConvertFromWChar(ColumnName,  NameLength4, &CpLength4, &Stmt->Connection->charset, NULL);
 
   ret= Stmt->Methods->Columns(Stmt, CpCatalog, (SQLSMALLINT)CpLength1, CpSchema, (SQLSMALLINT)CpLength2,
                               CpTable, (SQLSMALLINT)CpLength3, CpColumn, (SQLSMALLINT)CpLength4);
@@ -740,11 +740,11 @@ SQLRETURN SQL_API SQLConnectW(SQLHDBC ConnectionHandle,
 
    /* Convert parameters to Cp */
   if (ServerName)
-    MBServerName= MADB_ConvertFromWChar(ServerName, NameLength1, 0, Dbc->IsAnsi ? Dbc->ConnOrSrcCharset : &utf8, NULL);
+    MBServerName= MADB_ConvertFromWChar(ServerName, NameLength1, 0, &utf8, NULL);
   if (UserName)
-    MBUserName= MADB_ConvertFromWChar(UserName, NameLength2, 0, Dbc->IsAnsi ? Dbc->ConnOrSrcCharset : &utf8, NULL);
+    MBUserName= MADB_ConvertFromWChar(UserName, NameLength2, 0, &utf8, NULL);
   if (Authentication)
-    MBAuthentication= MADB_ConvertFromWChar(Authentication, NameLength3, 0, Dbc->IsAnsi ? Dbc->ConnOrSrcCharset : &utf8, NULL);
+    MBAuthentication= MADB_ConvertFromWChar(Authentication, NameLength3, 0, &utf8, NULL);
   
   ret= SQLConnectCommon(ConnectionHandle, (SQLCHAR *)MBServerName, SQL_NTS, (SQLCHAR *)MBUserName, SQL_NTS, 
                    (SQLCHAR *)MBAuthentication, SQL_NTS);
@@ -889,7 +889,7 @@ SQLRETURN SQL_API SQLDisconnect(SQLHDBC ConnectionHandle)
 {
   SQLRETURN ret= SQL_ERROR;
   MADB_Dbc *Connection = (MADB_Dbc *)ConnectionHandle;
-  LIST *Element, *NextElement;
+  MADB_List *Element, *NextElement;
 
   if (!Connection)
     return SQL_INVALID_HANDLE;
@@ -918,13 +918,11 @@ SQLRETURN SQL_API SQLDisconnect(SQLHDBC ConnectionHandle)
     mysql_close(Connection->mariadb);
     Connection->mariadb= NULL;
     ret= SQL_SUCCESS;
-  }
-  else
+  }  else
   {
     MADB_SetError(&Connection->Error, MADB_ERR_08003, NULL, 0);
     ret= Connection->Error.ReturnValue;
   }
-  Connection->ConnOrSrcCharset= NULL;
 
   MDBUG_C_RETURN(Connection, ret, &Connection->Error);
 }
@@ -986,7 +984,7 @@ SQLRETURN SQL_API SQLDriverConnectW(SQLHDBC      ConnectionHandle,
 
   MADB_CLEAR_ERROR(&Dbc->Error);
 
-  InConnStrA= MADB_ConvertFromWChar(InConnectionString, StringLength1, &InStrAOctLen, Dbc->IsAnsi ? Dbc->ConnOrSrcCharset : &utf8, NULL);
+  InConnStrA= MADB_ConvertFromWChar(InConnectionString, StringLength1, &InStrAOctLen, &utf8, NULL);
   MDBUG_C_DUMP(Dbc, Dbc, 0x);
   MDBUG_C_DUMP(Dbc, InConnStrA, s);
   MDBUG_C_DUMP(Dbc, StringLength1, d);
@@ -1071,7 +1069,7 @@ SQLRETURN MA_SQLEndTran(SQLSMALLINT HandleType,
   case SQL_HANDLE_ENV:
     {
       MADB_Env *Env= (MADB_Env *)Handle;
-      LIST *List= Env->Dbcs;
+      MADB_List *List= Env->Dbcs;
 
       for (List= Env->Dbcs; List; List= List->next)
         ((MADB_Dbc *)List->data)->Methods->EndTran((MADB_Dbc *)List->data, CompletionType);
@@ -1263,33 +1261,20 @@ SQLRETURN SQL_API SQLTransact(SQLHENV Env, SQLHDBC Dbc, SQLUSMALLINT CompletionT
 /* }}} */
 
 /* {{{ SQLExecDirect */
-SQLRETURN MA_SQLExecDirect(MADB_Stmt *Stmt,
+SQLRETURN SQL_API SQLExecDirect(SQLHSTMT StatementHandle,
     SQLCHAR *StatementText,
     SQLINTEGER TextLength)
 {
+  MADB_Stmt *Stmt= (MADB_Stmt *)StatementHandle;
   SQLRETURN ret;
 
   if (!Stmt)
     ret= SQL_INVALID_HANDLE;
   else
     ret= Stmt->Methods->ExecDirect(Stmt, (char *)StatementText, TextLength);
+
   MDBUG_C_RETURN(Stmt->Connection, ret, &Stmt->Error);
 }
-
-
-SQLRETURN SQL_API SQLExecDirect(SQLHSTMT StatementHandle,
-    SQLCHAR *StatementText,
-    SQLINTEGER TextLength)
-{
-  MADB_Stmt *Stmt= (MADB_Stmt *)StatementHandle;
-
-  MDBUG_C_ENTER(Stmt->Connection, "SQLExecDirect");
-  MDBUG_C_DUMP(Stmt->Connection, StatementHandle, 0x);
-  MDBUG_C_DUMP(Stmt->Connection, StatementText, s);
-  MDBUG_C_DUMP(Stmt->Connection, TextLength, d);
-
-  return MA_SQLExecDirect(Stmt, StatementText, TextLength);
-}
 /* }}} */
 
 /* {{{ SQLExecDirectW */
@@ -1312,7 +1297,7 @@ SQLRETURN SQL_API SQLExecDirectW(SQLHSTMT StatementHandle,
   MDBUG_C_ENTER(Stmt->Connection, "SQLExecDirectW");
   MDBUG_C_DUMP(Stmt->Connection, Stmt, 0x);
 
-  CpStmt= MADB_ConvertFromWChar(StatementText, TextLength, &StmtLength, Stmt->Connection->ConnOrSrcCharset, &ConversionError);
+  CpStmt= MADB_ConvertFromWChar(StatementText, TextLength, &StmtLength, &Stmt->Connection->charset, &ConversionError);
   MDBUG_C_DUMP(Stmt->Connection, CpStmt, s);
   if (ConversionError)
   {
@@ -1327,16 +1312,6 @@ SQLRETURN SQL_API SQLExecDirectW(SQLHSTMT StatementHandle,
 }
 /* }}} */
 
-/* {{{ SQLExecute */
-SQLRETURN MA_SQLExecute(MADB_Stmt *Stmt)
-{
-  SQLRETURN ret;
-
-  ret= Stmt->Methods->Execute(Stmt);
-
-  MDBUG_C_RETURN(Stmt->Connection, ret, &Stmt->Error);
-}
-
 SQLRETURN SQL_API SQLExecute(SQLHSTMT StatementHandle)
 {
   MADB_Stmt *Stmt= (MADB_Stmt *)StatementHandle;
@@ -1349,7 +1324,7 @@ SQLRETURN SQL_API SQLExecute(SQLHSTMT StatementHandle)
   MDBUG_C_ENTER(Stmt->Connection, "SQLExecute");
   MDBUG_C_DUMP(Stmt->Connection, Stmt, 0x);
 
-  return MA_SQLExecute(Stmt);
+  return Stmt->Methods->Execute(Stmt, FALSE);
 }
 /* }}} */
 
@@ -1378,7 +1353,7 @@ SQLRETURN SQL_API SQLExtendedFetch(SQLHSTMT StatementHandle,
    
   Stmt->Ird->Header.RowsProcessedPtr= RowCountPtr;
   Stmt->Ird->Header.ArrayStatusPtr= RowStatusArray;
-  ret= MA_SQLFetchScroll((SQLHSTMT)Stmt, FetchOrientation, FetchOffset);
+  ret=  Stmt->Methods->FetchScroll(Stmt, FetchOrientation, FetchOffset);
 
   if (RowStatusArray && SaveArrayStatusPtr)
   {
@@ -1406,41 +1381,39 @@ SQLRETURN SQL_API SQLExtendedFetch(SQLHSTMT StatementHandle,
 /* {{{ SQLFetch */
 SQLRETURN SQL_API SQLFetch(SQLHSTMT StatementHandle)
 {
+  MADB_Stmt *Stmt;
+  
   if (StatementHandle == SQL_NULL_HSTMT)
     return SQL_INVALID_HANDLE;
-  MDBUG_C_ENTER(((MADB_Stmt *)StatementHandle)->Connection, "SQLFetch");
+
+  Stmt= (MADB_Stmt *)StatementHandle;
+
+  MDBUG_C_ENTER(Stmt->Connection, "SQLFetch");
+  MADB_CLEAR_ERROR(&Stmt->Error);
+
   /* SQLFetch is equivalent of SQLFetchScroll(SQL_FETCH_NEXT), 3rd parameter is ignored for SQL_FETCH_NEXT */
-  return MA_SQLFetchScroll(StatementHandle, SQL_FETCH_NEXT, 1);
+  MDBUG_C_RETURN(Stmt->Connection, Stmt->Methods->FetchScroll(Stmt, SQL_FETCH_NEXT, 1), &Stmt->Error);
 }
 /* }}} */
 
 /* {{{ SQLFetchScroll */
-SQLRETURN MA_SQLFetchScroll(SQLHSTMT StatementHandle,
-    SQLSMALLINT FetchOrientation,
-    SQLLEN FetchOffset)
-{
-  MADB_Stmt *Stmt= (MADB_Stmt *)StatementHandle;
-  SQLRETURN ret;
-  if(!Stmt)
-    return SQL_INVALID_HANDLE;
-
-  MDBUG_C_ENTER(Stmt->Connection, "SQLFetchScroll");
-  MDBUG_C_DUMP(Stmt->Connection, FetchOrientation, d);
-
-  ret= Stmt->Methods->FetchScroll(Stmt, FetchOrientation, FetchOffset);
-
-  MDBUG_C_RETURN(Stmt->Connection, ret, &Stmt->Error);
-}
-
 SQLRETURN SQL_API SQLFetchScroll(SQLHSTMT StatementHandle,
     SQLSMALLINT FetchOrientation,
     SQLLEN FetchOffset)
 {
+  MADB_Stmt *Stmt;
+
   if (StatementHandle == SQL_NULL_HSTMT)
     return SQL_INVALID_HANDLE;
-  MADB_CLEAR_ERROR(&((MADB_Stmt*)StatementHandle)->Error);
 
-  return MA_SQLFetchScroll(StatementHandle, FetchOrientation, FetchOffset);
+  Stmt= (MADB_Stmt *)StatementHandle;
+
+  MDBUG_C_ENTER(Stmt->Connection, "SQLFetchScroll");
+  MDBUG_C_DUMP(Stmt->Connection, FetchOrientation, d);
+
+  MADB_CLEAR_ERROR(&Stmt->Error);
+
+  MDBUG_C_RETURN(Stmt->Connection, Stmt->Methods->FetchScroll(Stmt, FetchOrientation, FetchOffset), &Stmt->Error);
 }
 /* }}} */
 
@@ -1619,12 +1592,12 @@ SQLRETURN SQL_API SQLForeignKeysW(SQLHSTMT StatementHandle,
 
   MDBUG_C_ENTER(Stmt->Connection, "SQLForeignKeysW");
 
-  CpPkCatalog= MADB_ConvertFromWChar(PKCatalogName, NameLength1, &CpLength1, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpPkSchema= MADB_ConvertFromWChar(PKSchemaName, NameLength2, &CpLength2, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpPkTable= MADB_ConvertFromWChar(PKTableName, NameLength3, &CpLength3, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpFkCatalog= MADB_ConvertFromWChar(FKCatalogName, NameLength4, &CpLength4, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpFkSchema= MADB_ConvertFromWChar(FKSchemaName, NameLength5, &CpLength5, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpFkTable= MADB_ConvertFromWChar(FKTableName, NameLength6, &CpLength6, Stmt->Connection->ConnOrSrcCharset, NULL);
+  CpPkCatalog= MADB_ConvertFromWChar(PKCatalogName, NameLength1, &CpLength1, &Stmt->Connection->charset, NULL);
+  CpPkSchema= MADB_ConvertFromWChar(PKSchemaName, NameLength2, &CpLength2, &Stmt->Connection->charset, NULL);
+  CpPkTable= MADB_ConvertFromWChar(PKTableName, NameLength3, &CpLength3, &Stmt->Connection->charset, NULL);
+  CpFkCatalog= MADB_ConvertFromWChar(FKCatalogName, NameLength4, &CpLength4, &Stmt->Connection->charset, NULL);
+  CpFkSchema= MADB_ConvertFromWChar(FKSchemaName, NameLength5, &CpLength5, &Stmt->Connection->charset, NULL);
+  CpFkTable= MADB_ConvertFromWChar(FKTableName, NameLength6, &CpLength6, &Stmt->Connection->charset, NULL);
 
   ret= Stmt->Methods->ForeignKeys(Stmt, CpPkCatalog, (SQLSMALLINT)CpLength1, CpPkSchema, (SQLSMALLINT)CpLength2,
                                   CpPkTable, (SQLSMALLINT)CpLength3, CpFkCatalog, (SQLSMALLINT)CpLength4, 
@@ -2327,18 +2300,6 @@ SQLRETURN SQL_API SQLParamData(SQLHSTMT StatementHandle,
 }
 /* }}} */
 
-/* {{{ MA_SQLPrepare */
-SQLRETURN MA_SQLPrepare(MADB_Stmt *Stmt,
-    SQLCHAR *StatementText,
-    SQLINTEGER TextLength)
-{
-  SQLRETURN ret;
-  if (!Stmt)
-    return SQL_INVALID_HANDLE;
-  ret= Stmt->Methods->Prepare(Stmt, (char *)StatementText, TextLength);
-
-  MDBUG_C_RETURN(Stmt->Connection, ret, &Stmt->Error);
-}
 
 SQLRETURN SQL_API SQLPrepare(SQLHSTMT StatementHandle,
     SQLCHAR *StatementText,
@@ -2357,9 +2318,9 @@ SQLRETURN SQL_API SQLPrepare(SQLHSTMT StatementHandle,
   MDBUG_C_DUMP(Stmt->Connection, StatementText, s);
   MDBUG_C_DUMP(Stmt->Connection, TextLength, d);
 
-  MADB_CLEAR_ERROR(&Stmt->Error);
+  /* Prepare method clears error */
 
-  return MA_SQLPrepare(Stmt, StatementText, TextLength);
+  return Stmt->Methods->Prepare(Stmt, (char *)StatementText, TextLength, FALSE);
 }
 /* }}} */
 
@@ -2380,7 +2341,7 @@ SQLRETURN SQL_API SQLPrepareW(SQLHSTMT StatementHandle,
 
   MDBUG_C_ENTER(Stmt->Connection, "SQLPrepareW");
 
-  StmtStr= MADB_ConvertFromWChar(StatementText, TextLength, &StmtLength, Stmt->Connection->ConnOrSrcCharset, &ConversionError);
+  StmtStr= MADB_ConvertFromWChar(StatementText, TextLength, &StmtLength, &Stmt->Connection->charset, &ConversionError);
 
   MDBUG_C_DUMP(Stmt->Connection, Stmt, 0x);
   MDBUG_C_DUMP(Stmt->Connection, StmtStr, s);
@@ -2392,7 +2353,7 @@ SQLRETURN SQL_API SQLPrepareW(SQLHSTMT StatementHandle,
     ret= Stmt->Error.ReturnValue;
   }
   else
-    ret= Stmt->Methods->Prepare(Stmt, StmtStr, (SQLINTEGER)StmtLength);
+    ret= Stmt->Methods->Prepare(Stmt, StmtStr, (SQLINTEGER)StmtLength, FALSE);
   MADB_FREE(StmtStr);
 
   MDBUG_C_RETURN(Stmt->Connection, ret, &Stmt->Error);
@@ -2453,9 +2414,9 @@ SQLRETURN SQL_API SQLPrimaryKeysW(SQLHSTMT StatementHandle,
     return SQL_INVALID_HANDLE;
   MADB_CLEAR_ERROR(&Stmt->Error);
 
-  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpTable= MADB_ConvertFromWChar(TableName, NameLength3, &CpLength3, Stmt->Connection->ConnOrSrcCharset, NULL);
+  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, &Stmt->Connection->charset, NULL);
+  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, &Stmt->Connection->charset, NULL);
+  CpTable= MADB_ConvertFromWChar(TableName, NameLength3, &CpLength3, &Stmt->Connection->charset, NULL);
 
   MDBUG_C_ENTER(Stmt->Connection, "SQLPrimaryKeysW");
   MDBUG_C_DUMP(Stmt->Connection, StatementHandle, 0x);
@@ -2522,10 +2483,10 @@ SQLRETURN SQL_API SQLProcedureColumnsW(SQLHSTMT StatementHandle,
     return SQL_INVALID_HANDLE;
   MADB_CLEAR_ERROR(&Stmt->Error);
 
-  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpProc= MADB_ConvertFromWChar(ProcName, NameLength3, &CpLength3, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpColumn= MADB_ConvertFromWChar(ColumnName, NameLength4, &CpLength4, Stmt->Connection->ConnOrSrcCharset, NULL);
+  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, &Stmt->Connection->charset, NULL);
+  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, &Stmt->Connection->charset, NULL);
+  CpProc= MADB_ConvertFromWChar(ProcName, NameLength3, &CpLength3, &Stmt->Connection->charset, NULL);
+  CpColumn= MADB_ConvertFromWChar(ColumnName, NameLength4, &CpLength4, &Stmt->Connection->charset, NULL);
 
   ret= Stmt->Methods->ProcedureColumns(Stmt, CpCatalog, (SQLSMALLINT)CpLength1, CpSchema, (SQLSMALLINT)CpLength2,
                                        CpProc, (SQLSMALLINT)CpLength3, CpColumn, (SQLSMALLINT)CpLength4);
@@ -2578,9 +2539,9 @@ SQLRETURN SQL_API SQLProceduresW(SQLHSTMT StatementHandle,
     return SQL_INVALID_HANDLE;
   MADB_CLEAR_ERROR(&Stmt->Error);
 
-  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpProc= MADB_ConvertFromWChar(ProcName, NameLength3, &CpLength3, Stmt->Connection->ConnOrSrcCharset, NULL);
+  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, &Stmt->Connection->charset, NULL);
+  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, &Stmt->Connection->charset, NULL);
+  CpProc= MADB_ConvertFromWChar(ProcName, NameLength3, &CpLength3, &Stmt->Connection->charset, NULL);
   
   ret= Stmt->Methods->Procedures(Stmt, CpCatalog, (SQLSMALLINT)CpLength1, CpSchema, (SQLSMALLINT)CpLength2,
                                  CpProc, (SQLSMALLINT)CpLength3);
@@ -2755,7 +2716,7 @@ SQLRETURN SQL_API SQLSetCursorNameW(SQLHSTMT StatementHandle,
   }
   MADB_CLEAR_ERROR(&Stmt->Error);
 
-  CpName= MADB_ConvertFromWChar(CursorName, NameLength, &Length, Stmt->Connection->ConnOrSrcCharset, NULL);
+  CpName= MADB_ConvertFromWChar(CursorName, NameLength, &Length, &Stmt->Connection->charset, NULL);
   rc= Stmt->Methods->SetCursorName(Stmt, (char *)CpName, (SQLINTEGER)Length);
 
   MADB_FREE(CpName);
@@ -3024,9 +2985,9 @@ SQLRETURN SQL_API SQLSpecialColumnsW(SQLHSTMT StatementHandle,
     return SQL_INVALID_HANDLE;
   MADB_CLEAR_ERROR(&Stmt->Error);
 
-  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpTable= MADB_ConvertFromWChar(TableName, NameLength3, &CpLength3, Stmt->Connection->ConnOrSrcCharset, NULL);
+  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, &Stmt->Connection->charset, NULL);
+  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, &Stmt->Connection->charset, NULL);
+  CpTable= MADB_ConvertFromWChar(TableName, NameLength3, &CpLength3, &Stmt->Connection->charset, NULL);
 
   ret= Stmt->Methods->SpecialColumns(Stmt,IdentifierType, CpCatalog, (SQLSMALLINT)CpLength1, CpSchema, 
                                      (SQLSMALLINT)CpLength2, CpTable, (SQLSMALLINT)CpLength3, Scope, Nullable);
@@ -3081,9 +3042,9 @@ SQLRETURN SQL_API SQLStatisticsW(SQLHSTMT StatementHandle,
     return SQL_INVALID_HANDLE;
   MADB_CLEAR_ERROR(&Stmt->Error);
 
-  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, Stmt->Connection->ConnOrSrcCharset, NULL);
-  CpTable= MADB_ConvertFromWChar(TableName, NameLength3, &CpLength3, Stmt->Connection->ConnOrSrcCharset, NULL);
+  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, &Stmt->Connection->charset, NULL);
+  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, &Stmt->Connection->charset, NULL);
+  CpTable= MADB_ConvertFromWChar(TableName, NameLength3, &CpLength3, &Stmt->Connection->charset, NULL);
 
   if (!Stmt)
     return SQL_INVALID_HANDLE;
@@ -3128,25 +3089,23 @@ SQLRETURN SQL_API SQLTablePrivilegesW(SQLHSTMT StatementHandle,
   MADB_Stmt *Stmt= (MADB_Stmt *)StatementHandle;
   SQLRETURN ret;
   char *CpCatalog= NULL,
+       *CpSchema= NULL,
        *CpTable= NULL;
-  SQLULEN CpLength1= 0, CpLength3= 0;
+  SQLULEN CpLength1, CpLength2, CpLength3;
 
   if (!Stmt)
     return SQL_INVALID_HANDLE;
   MADB_CLEAR_ERROR(&Stmt->Error);
 
-  if (CatalogName != NULL)
-  {
-    CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, Stmt->Connection->ConnOrSrcCharset, NULL);
-  }  
-  if (TableName != NULL)
-  {
-    CpTable=   MADB_ConvertFromWChar(TableName, NameLength3, &CpLength3, Stmt->Connection->ConnOrSrcCharset, NULL);
-  }
+  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, &Stmt->Connection->charset, NULL);
+  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, &Stmt->Connection->charset, NULL);
+  CpTable= MADB_ConvertFromWChar(TableName, NameLength3, &CpLength3, &Stmt->Connection->charset, NULL);
 
-  ret= Stmt->Methods->TablePrivileges(Stmt, CpCatalog, (SQLSMALLINT)CpLength1, NULL, 0, CpTable, (SQLSMALLINT)CpLength3);
+  ret= Stmt->Methods->TablePrivileges(Stmt, CpCatalog, (SQLSMALLINT)CpLength1, CpSchema, (SQLSMALLINT)CpLength2,
+                                      CpTable, (SQLSMALLINT)CpLength3);
 
   MADB_FREE(CpCatalog);
+  MADB_FREE(CpSchema);
   MADB_FREE(CpTable);
   return ret;
 }
@@ -3189,29 +3148,17 @@ SQLRETURN SQL_API SQLTablesW(SQLHSTMT StatementHandle,
        *CpSchema= NULL,
        *CpTable= NULL,
        *CpType= NULL;
-  SQLULEN CpLength1= 0, CpLength2= 0, CpLength3= 0, CpLength4= 0;
+  SQLULEN CpLength1, CpLength2, CpLength3, CpLength4;
   SQLRETURN ret;
 
   if (!Stmt)
     return SQL_INVALID_HANDLE;
   MADB_CLEAR_ERROR(&Stmt->Error);
 
-  if (CatalogName)
-  {
-    CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, Stmt->Connection->ConnOrSrcCharset, NULL);
-  }
-  if (SchemaName)
-  {
-    CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, Stmt->Connection->ConnOrSrcCharset, NULL);
-  }
-  if (TableName)
-  {
-    CpTable= MADB_ConvertFromWChar(TableName, NameLength3, &CpLength3, Stmt->Connection->ConnOrSrcCharset, NULL);
-  }
-  if (TableType)
-  {
-    CpType= MADB_ConvertFromWChar(TableType, NameLength4, &CpLength4, Stmt->Connection->ConnOrSrcCharset, NULL);
-  }
+  CpCatalog= MADB_ConvertFromWChar(CatalogName, NameLength1, &CpLength1, &Stmt->Connection->charset, NULL);
+  CpSchema= MADB_ConvertFromWChar(SchemaName, NameLength2, &CpLength2, &Stmt->Connection->charset, NULL);
+  CpTable= MADB_ConvertFromWChar(TableName, NameLength3, &CpLength3, &Stmt->Connection->charset, NULL);
+  CpType= MADB_ConvertFromWChar(TableType, NameLength4, &CpLength4, &Stmt->Connection->charset, NULL);
 
   ret= Stmt->Methods->Tables(Stmt, CpCatalog, (SQLSMALLINT)CpLength1, CpSchema, (SQLSMALLINT)CpLength2, 
                                CpTable, (SQLSMALLINT)CpLength3, CpType, (SQLSMALLINT)CpLength4);
